# etpgrf/conf.py
# Настройки по умолчанию и "источник правды" для типографа etpgrf
from html import entities

# === КОНФИГУРАЦИИ ===
# Режимы "отдачи" результатов обработки
MODE_UNICODE = "unicode"
MODE_MNEMONIC = "mnemonic"
MODE_MIXED = "mixed"

# Языки, поддерживаемые библиотекой
LANG_RU = 'ru'  # Русский
LANG_RU_OLD = 'ruold'  # Русская дореволюционная орфография
LANG_EN = 'en'  # Английский
SUPPORTED_LANGS = frozenset([LANG_RU, LANG_RU_OLD, LANG_EN])
DEFAULT_LANGS = (LANG_RU, LANG_EN)  # Языки по умолчанию

# Виды санитизации (очистки) входного текста
SANITIZE_ALL_HTML = "html"        # Полная очистка от HTML-тегов
SANITIZE_ETPGRF = "etp"           # Очистка от "span-оберток" символов висячей пунктуации (если она была расставлена
                                  # при предыдущих проходах типографа)
SANITIZE_NONE = None              # Без очистки (режим по умолчанию). False тоже можно использовать.

# === ИСТОЧНИК ПРАВДЫ ===
# --- Базовые алфавиты: Эти константы используются как для правил переноса, так и для правил кодирования ---

# Русский алфавит
RU_VOWELS_UPPER = frozenset(['А', 'О', 'И', 'Е', 'Ё', 'Э', 'Ы', 'У', 'Ю', 'Я'])
RU_CONSONANTS_UPPER = frozenset(['Б', 'В', 'Г', 'Д', 'Ж', 'З', 'К', 'Л', 'М', 'Н', 'П', 'Р', 'С', 'Т', 'Ф', 'Х', 'Ц', 'Ч', 'Ш', 'Щ'])
RU_J_SOUND_UPPER = frozenset(['Й'])
RU_SIGNS_UPPER = frozenset(['Ь', 'Ъ'])
RU_ALPHABET_UPPER = RU_VOWELS_UPPER | RU_CONSONANTS_UPPER | RU_J_SOUND_UPPER | RU_SIGNS_UPPER
RU_ALPHABET_LOWER = frozenset([char.lower() for char in RU_ALPHABET_UPPER])
RU_ALPHABET_FULL = RU_ALPHABET_UPPER | RU_ALPHABET_LOWER

# Английский алфавит
EN_VOWELS_UPPER = frozenset(['A', 'E', 'I', 'O', 'U', 'Æ', 'Œ'])
EN_CONSONANTS_UPPER = frozenset(['B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'X', 'Y', 'Z'])
EN_ALPHABET_UPPER = EN_VOWELS_UPPER | EN_CONSONANTS_UPPER
EN_ALPHABET_LOWER = frozenset([char.lower() for char in EN_ALPHABET_UPPER])
EN_ALPHABET_FULL = EN_ALPHABET_UPPER | EN_ALPHABET_LOWER

# --- Специальные символы ---
CHAR_NBSP = '\u00a0'      # Неразрывный пробел (&nbsp;)
CHAR_SHY = '\u00ad'       # Мягкий перенос (&shy;)
CHAR_THIN_SP = '\u2009'   # Тонкий пробел (шпация, &thinsp;)
CHAR_NDASH = '\u2013'     # Cреднее тире (– / &ndash;)
CHAR_MDASH = '\u2014'     # Длинное тире (—  / &mdash;)
CHAR_HELLIP = '\u2026'    # Многоточие (… / &hellip;)
CHAR_RU_QUOT1_OPEN = '«'  # Русские кавычки открывающие (« / &laquo;)
CHAR_RU_QUOT1_CLOSE = '»'
CHAR_RU_QUOT2_OPEN = '„'
CHAR_RU_QUOT2_CLOSE = '“'
CHAR_EN_QUOT1_OPEN = '“'
CHAR_EN_QUOT1_CLOSE = '”'
CHAR_EN_QUOT2_OPEN = '‘'
CHAR_EN_QUOT2_CLOSE = '’'
CHAR_COPY = '\u00a9'  # Символ авторского права / © / &copy;
CHAR_REG = '\u00ae'   # Зарегистрированная торговая марка / ® / &reg;
CHAR_COPYP = '\u2117' # Знак звуковой записи / ℗ / &copyp;
CHAR_TRADE = '\u2122' # Знак торговой марки / ™ / &trade;
CHAR_ARROW_LR_DOUBLE = '\u21d4' # Двойная двунаправленная стрелка / ⇔ / &hArr;
CHAR_ARROW_L_DOUBLE = '\u21d0'  # Двойная стрелка влево / ⇐ / &lArr;
CHAR_ARROW_R_DOUBLE = '\u21d2'  # Двойная стрелка вправо / ⇒ / &rArr;
CHAR_AP = '\u2248'         # Приблизительно равно / ≈ / &ap;
CHAR_ARROW_L = '\u27f5'    # Стрелка влево / ← / &larr;
CHAR_ARROW_R = '\u27f6'    # Стрелка вправо / → / &rarr;
CHAR_ARROW_LR = '\u27f7'   # Длинная двунаправленная стрелка ↔ / &harr;
CHAR_ARROW_L_LONG_DOUBLE = '\u27f8'  # Длинная двойная стрелка влево
CHAR_ARROW_R_LONG_DOUBLE = '\u27f9'  # Длинная двойная стрелка вправо
CHAR_ARROW_LR_LONG_DOUBLE = '\u27fa' # Длинная двойная двунаправленная стрелка
CHAR_MIDDOT = '\u00b7'    # Средняя точка (· иногда используется как знак умножения) / &middot;
CHAR_UNIT_SEPARATOR = '\u25F0' # Символ временный разделитель для составных единиц (◰), чтобы не уходить
                               # в "мертвый" цикл при замене на тонкий пробел. Можно взять любой редкий символом.


# === КОНСТАНТЫ ПСЕВДОГРАФИКИ ===
# Для простых замен "строка -> символ" используем список кортежей.
# Порядок важен: более длинные последовательности должны идти раньше более коротких, которые
#                могут быть их частью (например, '<---' до '---', а та, в свою очередь, до '--').
STR_TO_SYMBOL_REPLACEMENTS = [
    # 5-символьные последовательности
    ('<===>', CHAR_ARROW_LR_LONG_DOUBLE),  # Длинная двойная двунаправленная стрелка
    # 4-символьные последовательности
    ('<===', CHAR_ARROW_L_LONG_DOUBLE),  # Длинная двойная стрелка влево
    ('===>', CHAR_ARROW_R_LONG_DOUBLE),  # Длинная двойная стрелка вправо
    ('<==>', CHAR_ARROW_LR_DOUBLE),  # Двойная двунаправленная стрелка
    ('(tm)', CHAR_TRADE), ('(TM)', CHAR_TRADE), # Знак торговой марки (нижний и верхний регистр)
    ('<-->', CHAR_ARROW_LR),  # Длинная двунаправленная стрелка
    # 3-символьные последовательности
    ('<--', CHAR_ARROW_L),  # Стрелка влево
    ('-->', CHAR_ARROW_R),  # Стрелка вправо
    ('==>', CHAR_ARROW_R_DOUBLE),  # Двойная стрелка вправо
    ('<==', CHAR_ARROW_L_DOUBLE),  # Двойная стрелка влево
    ('---', CHAR_MDASH), # Длинное тире
    ('...', CHAR_HELLIP),  # Многоточие
    ('(c)', CHAR_COPY), ('(C)', CHAR_COPY), # Знак авторского права (нижний и верхний регистр)
    ('(r)', CHAR_REG), ('(R)', CHAR_REG), # Знак зарегистрированной торговой марки (нижний и верхний регистр)
    ('(p)', CHAR_COPYP), ('(P)', CHAR_COPYP), # Знак права на звукозапись (нижний и верхний регистр)
    # 2-символьные последовательности
    ('--', CHAR_NDASH), # Среднее тире (дефисные соединения и диапазоны)
    ('~=', CHAR_AP),  # Приблизительно равно (≈)
]


# === КОНСТАНТЫ ДЛЯ КОДИРОВАНИЯ HTML-МНЕМНОИКОВ ===
# --- ЧЕРНЫЙ СПИСОК: Символы, которые НИКОГДА не нужно кодировать в мнемоники ---
NEVER_ENCODE_CHARS = (frozenset(['!', '#', '%', '(', ')', '*', ',', '.', '/', ':', ';', '=', '?', '@',
                                 '[', '\\', ']', '^', '_', '`', '{', '|', '}', '~', '\n', '\t', '\r'])
                      | RU_ALPHABET_FULL | EN_ALPHABET_FULL)

# 2. БЕЛЫЙ СПИСОК (ДЛЯ БЕЗОПАСНОСТИ):
#    Символы, которые ВСЕГДА должны превращаться в мнемоники в "безопасных" режимах вывода. Сюда добавлены символы,
#    которые не видны, на глаз и не отличимы друг от друга в обычном тексте, или очень специфичные
SAFE_MODE_CHARS_TO_MNEMONIC = frozenset([
    '<', '>', '&', '"', '\'',
    CHAR_SHY,  # Мягкий перенос (Soft Hyphen) -- &shy;
    CHAR_NBSP, # Неразрывный пробел (Non-Breaking Space) -- &nbsp;
    '\u2002',  # Полужирный пробел (En Space) -- &ensp;
    '\u2003',  # Широкий пробел (Em Space) -- &emsp;
    '\u2007',  # Цифровой пробел -- &numsp;
    '\u2008',  # Пунктуационный пробел -- &puncsp;
    CHAR_THIN_SP,  # Межсимвольный пробел, тонкий пробел, шпация -- &thinsp;'
    '\u200A',  # Толщина волоса (Hair Space) -- &hairsp;
    '\u200B',  # Негативный пробел (Negative Space) -- &NegativeThinSpace;
    '\u200C',  # Нулевая ширина (без объединения) (Zero Width Non-Joiner) -- &zwj;
    '\u200D',  # Нулевая ширина (с объединением) (Zero Width Joiner) -- &zwnj;
    '\u200E',  # Изменить направление текста на слева-направо (Left-to-Right Mark /LRE) -- &lrm;
    '\u200F',  # Изменить направление текста направо-налево (Right-to-Left Mark /RLM) -- &rlm;
    '\u2010',  # Дефис (Hyphen) -- &dash;
    '\u205F',  # Средний пробел (Medium Mathematical Space) -- &MediumSpace;
    '\u2060',  # &NoBreak;
    '\u2062',  # &InvisibleTimes; -- для семантической разметки математических выражений
    '\u2063',  # &InvisibleComma; -- для семантической разметки математических выражений
    ])

# 3. СПИСОК ДЛЯ ЧИСЛОВОГО КОДИРОВАНИЯ: Символы без стандартного имени.
ALWAYS_ENCODE_TO_NUMERIC_CHARS = frozenset([
    '\u058F',  # Знак армянского драма (֏)
    '\u20B4',  # Знак украинской гривны (₴)
    '\u20B8',  # Знак казахстанского тенге (₸)
    '\u20B9',  # Знак индийской рупии (₹)
    '\u20BA',  # Знак турецкой лиры (₺)
    '\u20BB',  # Знак итальянской лиры (₻)
    '\u20BC',  # Знак азербайджанского маната
    '\u20BD',  # Знак русского рубля (₽)
    '\u20BE',  # Знак грузинский лари (₾)
    '\u20BF',  # Знак биткоина (₿)
])

# 4. СЛОВАРЬ ПРИОРИТЕТОВ: Кастомные и/или предпочитаемые мнемоники.
#    Некоторые utf-символы имеют несколько мнемоник, а значит для таких символов преобразование
#    в из utf во html-мнемоники может иметь несколько вариантов. Словарь приоритетов задает предпочтительное
#    преобразование. Эти правила применяются в последнюю очередь и имеют наивысший приоритет,
#    гарантируя предсказуемый результат для символов с несколькими именами.
#
#    Также можно использовать для создания исключений из "черного списка" NEVER_ENCODE_CHARS.
CUSTOM_ENCODE_MAP = {
    # '\u2010': '&hyphen;',  # Для \u2010 всегда предпочитаем &hyphen;, а не &dash;
    # # Исключения для букв, которые есть в алфавитах, но должны кодироваться (для обеспечения консистентности):
    # # 'Æ': '&AElig;',
    # # 'Œ': '&OElig;',
    # # 'æ': '&aelig;',
    # # 'œ': '&oelig;',
    # '\u002a': '&ast;',		# * / &ast; / &midast;
    # '\u005b': '&lsqb;',       # [ / &lsqb; / &lbrack;
    # '\u005d': '&rsqb;',       # ] / &rsqb; / &rbrack;
    # '\u005f': '&lowbar;',     # _ / &lowbar; / &UnderBar;
    # '\u007b': '&lcub;',       # { / &lcub; / &lbrace;
    # '\u007d': '&rcub;',       # } / &rcub; / &rbrace;
    # '\u007c': '&vert;',       # | / &vert; / &verbar; / &VerticalLine;
    CHAR_NBSP: '&nbsp;',      #   / &nbsp; / &NonBreakingSpace;
    CHAR_REG: '&reg;',  # ® / &reg; / &REG; / &circledR;
    CHAR_COPY: '&copy;',  # © / &copy; / &COPY;
    '\u0022': '&quot;',         # " / &quot; / &QUOT;
    '\u0026': '&amp;',          # & / &amp; / &AMP;
    '\u003e': '&gt;',           # > / &gt; / &GT;
    '\u003c': '&lt;',           # < / &lt; / &LT;
    CHAR_MIDDOT: '&middot;',    # · / &middot; / &centerdot; / &CenterDot;
    '\u0060': '&grave;',        # ` / &grave; / &DiacriticalGrave;
    '\u00a8': '&die;',          # ¨ / &die; / &Dot; / &uml; / &DoubleDot;
    '\u00b1': '&pm;',           # ± / &pm; / &PlusMinus;
    '\u00bd': '&half;',         # ½ / &frac12; / &half;
    '\u00af': '&macr;',         # ¯ / &macr; / &strns;
    '\u201a': '&sbquo;',        # ‚ / &sbquo; / &lsquor;
    '\u223e': '&ac;',		    # ∾ / &ac; / &mstpos;
    '\u2207': '&Del;',		    # ∇ / &Del; / &nabla;
    '\u2061': '&af;',		    #   / &af; / &ApplyFunction;
    '\u2221': '&angmsd;',		# ∡ / &angmsd; / &measuredangle;
    CHAR_AP: '&ap;',		    # ≈ / &ap; / &thkap; / &approx; / &TildeTilde; / &thickapprox;
    '\u224a': '&ape;',		    # ≊ / &ape; / &approxeq;
    '\u2254': '&Assign;',		# ≔ / &Assign; / &colone; / &coloneq;
    '\u224d': '&CupCap;',		# ≍ / &CupCap; / &asympeq;
    '\u2233': '&awconint;',		# ∳ / &awconint; / &CounterClockwiseContourIntegral;
    '\u224c': '&bcong;',		# ≌ / &bcong; / &backcong;
    '\u03f6': '&bepsi;',		# ϶ / &bepsi; / &backepsilon;
    '\u2035': '&bprime;',		# ‵ / &bprime; / &backprime;
    '\u223d': '&bsim;',	    	# ∽ / &bsim; / &backsim;
    '\u22cd': '&bsime;',		# ⋍ / &bsime; / &backsimeq;
    '\u2216': '&setmn;',		# ∖ / &setmn; / &ssetmn; / &setminus; / &Backslash; / &smallsetminus;
    '\u2306': '&Barwed;',		# ⌆ / &Barwed; / &doublebarwedge;
    '\u2305': '&barwed;',		# ⌅ / &barwed; / &barwedge;
    '\u23b5': '&bbrk;',		    # ⎵ / &bbrk; / &UnderBracket;
    '\u2235': '&becaus;',		# ∵ / &becaus; / &because; / &Because;
    '\u212c': '&Bscr;',		    # ℬ / &Bscr; / &bernou; / &Bernoullis;
    '\u2264': '&le;',		    # ≤ / &le; / &leq;
    '\u226c': '&twixt;',		# ≬ / &twixt; / &between;
    '\u22c2': '&xcap;',		    # ⋂ / &xcap; / &bigcap; / &Intersection;
    '\u25ef': '&xcirc;',		# ◯ / &xcirc; / &bigcirc;
    '\u22c3': '&xcup;',		    # ⋃ / &xcup; / &Union; / &bigcup;
    '\u2a00': '&xodot;',		# ⨀ / &xodot; / &bigodot;
    '\u2a01': '&xoplus;',		# ⨁ / &xoplus; / &bigoplus;
    '\u2a02': '&xotime;',		# ⨂ / &xotime; / &bigotimes;
    '\u2a06': '&xsqcup;',		# ⨆ / &xsqcup; / &bigsqcup;
    '\u2605': '&starf;',		# ★ / &starf; / &bigstar;
    '\u25bd': '&xdtri;',		# ▽ / &xdtri; / &bigtriangledown;
    '\u25b3': '&xutri;',		# △ / &xutri; / &bigtriangleup;
    '\u2a04': '&xuplus;',		# ⨄ / &xuplus; / &biguplus;
    '\u22c1': '&Vee;',		    # ⋁ / &Vee; / &xvee; / &bigvee;
    '\u22c0': '&Wedge;',		# ⋀ / &Wedge; / &xwedge; / $bigwedge;
    '\u2227': '&and;',		    # ∧ / &and; / &wedge;
    '\u290d': '&rbarr;',		# ⤍ / &rbarr; / &bkarow;
    '\u29eb': '&lozf;',		    # ⧫ / &lozf; / &blacklozenge;
    '\u25ca': '&loz;',		    # ◊ / &loz; / &lozenge
    '\u25aa': '&squf;',		    # ▪ / &squf; / &squarf; / &blacksquare; / &FilledVerySmallSquare;
    '\u25b4': '&utrif;',		# ▴ / &utrif; / &blacktriangle;
    '\u25be': '&dtrif;',		# ▾ / &dtrif; / &blacktriangledown;
    '\u25c2': '&ltrif;',		# ◂ / &ltrif; / &blacktriangleleft;
    '\u25b8': '&rtrif;',		# ▸ / &rtrif; / &blacktriangleright;
    '\u22a5': '&bot;',		    # ⊥ / &bot; / &UpTee; / &bottom; / &perp;
    '\u2500': '&boxh;',		    # ─ / &boxh; / &HorizontalLine;
    '\u229f': '&minusb;',		# ⊟ / &minusb; / &boxminus;
    '\u229e': '&plusb;',		# ⊞ / &plusb; / &boxplus;
    '\u22a0': '&timesb;',		# ⊠ / &timesb; / &boxtimes;
    '\u02d8': '&breve;',		# ˘ / &breve; / &Breve;
    '\u224e': '&bump;',		    # ≎ / &bump; / &Bumpeq; / &HumpDownHump;
    '\u224f': '&bumpe;',		# ≏ / &bumpe; / &bumpeq; / &HumpEqual;
    '\u2145': '&DD;',		    # ⅅ / &DD; / &CapitalDifferentialD;
    '\u02c7': '&caron;',		# ˇ / &Hacek; / &caron;
    '\u212d': '&Cfr;',		    # ℭ / &Cfr; / &Cayleys;
    '\u2713': '&check;',		# ✓ / &check; / &checkmark;
    '\u2257': '&cire;',		    # ≗ / &cire; / &circeq;
    '\u21ba': '&olarr;',		# ↺ / &olarr; / &circlearrowleft;
    '\u21bb': '&orarr;',		# ↻ / &orarr; / &circlearrowright;
    '\u229b': '&oast;',		    # ⊛ / &oast; / &circledast;
    '\u229a': '&ocir;',		    # ⊚ / &ocir; / &circledcirc;
    '\u229d': '&odash;',		# ⊝ / &odash; / &circleddash;
    '\u2299': '&odot;',		    # ⊙ / &odot; / &CircleDot;
    '\u2200': '&forall;',		# ∀ / &forall; / &ForAll;
    '\u24c8': '&oS;',		    # Ⓢ / &oS; / &circledS;
    '\u2296': '&ominus;',		# ⊖ / &ominus; / &CircleMinus;
    '\u2232': '&cwconint;',		# ∲ / &cwconint; / &ClockwiseContourIntegral;
    '\u201d': '&rdquo;',		# ” /  &rdquo; / &rdquor; / &CloseCurlyDoubleQuote;
    '\u2019': '&rsquo;',		# ’ / &rsquo; / &rsquor; / &CloseCurlyQuote;
    '\u2237': '&Colon;',		# ∷ / &Colon; / &Proportion;
    '\u2201': '&comp;',		    # ∁ / &comp; / &complement;
    '\u2218': '&compfn;',		# ∘ / &compfn; / &SmallCircle;
    '\u2102': '&Copf;',		    # ℂ / &Copf; / &complexes;
    '\u222f': '&Conint;',		# ∯ / &Conint; / &DoubleContourIntegral;
    '\u222e': '&oint;',		    # ∮ / &oint; / &conint; / &ContourIntegral;
    '\u2210': '&coprod;',		# ∐ / &coprod; / &Coproduct;
    '\u22de': '&cuepr;',		# ⋞ / &cuepr; / &curlyeqprec;
    '\u22df': '&cuesc;',		# ⋟ / &cuesc; / &curlyeqsucc;
    '\u21b6': '&cularr;',		# ↶ / &cularr; / &curvearrowleft;
    '\u21b7': '&curarr;',		# ↷ / &curarr; / &curvearrowright;
    '\u22ce': '&cuvee;',		# ⋎ / &cuvee; / &curlyvee;
    '\u22cf': '&cuwed;',		# ⋏ / &cuwed; / &curlywedge;
    '\u2010': '&dash;',		    # ‐ / &dash; / &hyphen;
    '\u2ae4': '&Dashv;',		# ⫤ / &Dashv; / &DoubleLeftTee;
    '\u22a3': '&dashv;',		# ⊣ / &dashv; / &LeftTee;
    '\u290f': '&rBarr;',		# ⤏ / &rBarr; / &dbkarow;
    '\u02dd': '&dblac;',		# ˝ / &dblac; / &DiacriticalDoubleAcute;
    '\u2146': '&dd;',		    # ⅆ / &dd; / &DifferentialD;
    '\u21ca': '&ddarr;',		# ⇊ / &ddarr; / &downdownarrows;
    '\u2a77': '&eDDot;',		# ⩷ / &eDDot; / &ddotseq;
    '\u21c3': '&dharl;',		# ⇃ / &dharl; / &LeftDownVector; / &downharpoonleft;
    '\u21c2': '&dharr;',		# ⇂ / &dharr; / &RightDownVector; / &downharpoonright;
    '\u02d9': '&dot;',		    # ˙ / &dot; / &DiacriticalDot;
    '\u222b': '&int;',		    # ∫ / &int; / &Integral;
    '\u22c4': '&diam;',		    # ⋄ / &diam; / &diamond; / &Diamond;
    '\u03b5': '&epsi;',		    # ε / &epsi; / &epsilon;
    '\u03dd': '&gammad;',		# ϝ / &gammad; / &digamma;
    '\u22c7': '&divonx;',		# ⋇ / &divonx; / &divideontimes;
    '\u231e': '&dlcorn;',		# ⌞ / &dlcorn; / &llcorner;
    '\u2250': '&esdot;',		# ≐ / &esdot; / &doteq; / &DotEqual;
    '\u2251': '&eDot;',		    # ≑ / &eDot; / &doteqdot;
    '\u2238': '&minusd;',		# ∸ / &minusd; / &dotminus;
    '\u2214': '&plusdo;',		# ∔ / &plusdo; / &dotplus;
    '\u22a1': '&sdotb;',		# ⊡ / &sdotb; / &dotsquare;
    '\u21d3': '&dArr;',		    # ⇓ / &dArr; / &Downarrow; / &DoubleDownArrow;
    CHAR_ARROW_R_DOUBLE: '&rArr;',  # ⇒ / &rArr; / &Implies; / &Rightarrow; / &DoubleRightArrow;
    CHAR_ARROW_L_DOUBLE: '&lArr;',   # ⇐ / &lArr; / &Leftarrow; / &DoubleLeftArrow;
    CHAR_ARROW_LR_DOUBLE: '&iff;',   # ⇔ / &iff; / &hArr; / &Leftrightarrow; / &DoubleLeftRightArrow;
    CHAR_ARROW_L_LONG_DOUBLE: '&xlArr;',		# ⟸ / &xlArr; / &Longleftarrow; / &DoubleLongLeftArrow;
    CHAR_ARROW_R_LONG_DOUBLE: '&xrArr;',  # ⟹ / &xrArr; / &Longrightarrow; / &DoubleLongRightArrow;
    CHAR_ARROW_LR_LONG_DOUBLE: '&xhArr;',		# ⟺ / &xhArr; / &Longleftrightarrow; / &DoubleLongLeftRightArrow;
    '\u22a8': '&vDash;',		# ⊨ / &vDash; / &DoubleRightTee;
    '\u21d1': '&uArr;',		    # ⇑ / &uArr; / &Uparrow; / &DoubleUpArrow;
    '\u2202': '&part;',         # ∂ / &part; / &PartialD;
    '\u21d5': '&vArr;',		    # ⇕ / &vArr; / &Updownarrow; / &DoubleUpDownArrow;
    '\u2225': '&par;',		    # ∥ / &par; / &spar; / &parallel; / &shortparallel; / &DoubleVerticalBar;
    '\u2193': '&darr;',	   	    # ↓ / &darr; / &downarrow; / &DownArrow; / &ShortDownArrow;
    '\u21f5': '&duarr;',		# ⇵ / &duarr; / &DownArrowUpArrow;
    '\u21bd': '&lhard;',		# ↽ / &lhard; /&DownLeftVector; / &leftharpoondown;
    '\u21c1': '&rhard;',		# ⇁ / &rhard; / &DownRightVector; / &rightharpoondown;
    '\u22a4': '&top;',		    # ⊤ / &top; / &DownTee;
    '\u21a7': '&mapstodown;',	# ↧ / &mapstodown; / &DownTeeArrow;
    '\u2910': '&RBarr;',		# ⤐ / &RBarr; / &drbkarow;
    '\u231f': '&drcorn;',		# ⌟ / &drcorn; / &lrcorner;
    '\u25bf': '&dtri;',		    # ▿ / &dtri; / &triangledown;
    '\u296f': '&duhar;',		# ⥯ / &duhar; / &ReverseUpEquilibrium;
    '\u2256': '&ecir;',		    # ≖ / &ecir; / &eqcirc;
    '\u2255': '&ecolon;',		# ≕ / &ecolon; / &eqcolon;
    '\u2147': '&ee;',		    # ⅇ / &ee; / &exponentiale; / &ExponentialE;
    '\u2252': '&efDot;',		# ≒ / &efDot; / &fallingdotseq;
    '\u2a96': '&egs;',		    # ⪖ / &egs; / &eqslantgtr;
    '\u2208': '&in;',		    # ∈ / &in; / &isin; / &isinv; / &Element;
    '\u2a95': '&els;',		    # ⪕ / &els; / &eqslantless;
    '\u2205': '&empty;',		# ∅ / &empty; / &emptyv; / &emptyset; / &varnothing;
    '\u03f5': '&epsiv;',		# ϵ / &epsiv; / &varepsilon; / &straightepsilon;
    '\u2242': '&esim;',		    # ≂ / &esim; / &eqsim; / &EqualTilde;
    '\u225f': '&equest;',		# ≟ / &equest; / &questeq;
    '\u21cc': '&rlhar;',		# ⇌ / &rlhar; / &Equilibrium; / &rightleftharpoons;
    '\u2253': '&erDot;',		# ≓ / &erDot; / &risingdotseq;
    '\u2130': '&Escr;',		    # ℰ / &Escr; / &expectation;
    '\u22d4': '&fork;',		    # ⋔ / &fork; / &pitchfork;
    '\u2131': '&Fscr;',		    # ℱ / &Fscr; / &Fouriertrf;
    '\u2322': '&frown;',		# ⌢ / &frown; / &sfrown;
    '\u2a86': '&gap;',		    # ⪆ / &gap; / &gtrapprox;
    '\u2267': '&gE;',		    # ≧ / &gE; / &geqq; / &GreaterFullEqual;
    '\u2a8c': '&gEl;',	  	    # ⪌ / &gEl; / &gtreqqless;
    '\u22db': '&gel;',		    # ⋛ / &gel; / &gtreqless; / &GreaterEqualLess;
    '\u2265': '&ge;',		    # ≥ / &ge; / &geq; / &GreaterEqual;
    '\u2a7e': '&ges;',		    # ⩾ / &ges; / &geqslant; / &GreaterSlantEqual;
    '\u22d9': '&Gg;',		    # ⋙ / &Gg; / &ggg;
    '\u226b': '&gg;',		    # ≫ / &gg ;/ &Gt; / &NestedGreaterGreater;
    '\u2277': '&gl;',		    # ≷ / &gl; / &gtrless; / &GreaterLess;
    '\u2a8a': '&gnap;',		    # ⪊ / &gnap; / &gnapprox;
    '\u2269': '&gnE;',		    # ≩ / &gnE; / &gneqq;
    '\u2260': '&ne;',		    # ≠ / &ne; / &NotEqual;
    '\u2a88': '&gne;',		    # ⪈ / &gne; / &gneq;
    '\u2273': '&gsim;',		    # ≳ / &gsim; / &gtrsim; / &GreaterTilde;
    '\u22d7': '&gtdot;',		# ⋗ / &gtdot; / &gtrdot;
    '\u200a': '&hairsp;',		#   / &hairsp; / &VeryThinSpace;
    '\u210b': '&Hscr;',		    # ℋ / &Hscr; / &hamilt; / &HilbertSpace;
    '\u21ad': '&harrw;',		# ↭ / &harrw; / &leftrightsquigarrow;
    '\u210f': '&hbar;',		    # ℏ / &hbar; / &planck; / &hslash; / &plankv;
    '\u210c': '&Hfr;',		    # ℌ / &Hfr; / &Poincareplane;
    '\u2925': '&searhk;',		# ⤥ / &searhk; / &hksearow;
    '\u2926': '&swarhk;',		# ⤦ / &swarhk; / &hkswarow;
    '\u21a9': '&larrhk;',		# ↩ / &larrhk; / &hookleftarrow;
    '\u21aa': '&rarrhk;',		# ↪ / &rarrhk; / &hookrightarrow;
    '\u210d': '&Hopf;',		    # ℍ / &Hopf; / &quaternions;
    '\u2063': '&ic;',		    # ⁣ / &ic; / &InvisibleComma;
    '\u2111': '&Im;',		    # ℑ / &Im; / &Ifr; / &image; / &imagpart;
    '\u2148': '&ii;',		    # ⅈ / &ii; / &ImaginaryI;
    '\u2a0c': '&qint;',		    # ⨌ / &qint; / &iiiint;
    '\u222d': '&tint;',		    # ∭ / &tint; / &iiint;
    '\u2110': '&Iscr;',		    # ℐ / &Iscr; / &imagline;
    '\u0131': '&imath;',		# ı / &imath; / &inodot;
    '\u22ba': '&intcal;',		# ⊺ / &intcal; / &intercal;
    '\u2124': '&Zopf;',		    # ℤ / &Zopf; / &integers;
    '\u2a3c': '&iprod;',		# ⨼ / &iprod; / &intprod;
    '\u2062': '&it;',		    # ⁢ / &it; / &InvisibleTimes;
    '\u03f0': '&kappav;',		# ϰ / &kappav; / &varkappa;
    '\u21da': '&lAarr;',		# ⇚ / &lAarr; / &Lleftarrow;
    '\u2112': '&Lscr;',		    # ℒ / &Lscr; / &lagran; / &Laplacetrf;
    '\u27e8': '&lang;',		    # ⟨ / &lang; / &langle; / &LeftAngleBracket;
    '\u2a85': '&lap;',		    # ⪅ / &lap; / &lessapprox;
    '\u219e': '&Larr;',		    # ↞ / &Larr; / &twoheadleftarrow;
    '\u21e4': '&larrb;',		# ⇤ / &larrb; / &LeftArrowBar;
    '\u21ab': '&larrlp;',		# ↫ / &larrlp; / &looparrowleft;
    '\u21a2': '&larrtl;',		# ↢ / &larrtl; / &leftarrowtail;
    '\u2266': '&lE;',		    # ≦ / &lE; / &leqq; / &LessFullEqual;
    '\u2190': '&larr;',		    # ← / &larr; / &slarr; / &LeftArrow; / &leftarrow; / &ShortLeftArrow;
    '\u21c6': '&lrarr;',		# ⇆ / &lrarr; / &leftrightarrows; / &LeftArrowRightArrow;
    '\u27e6': '&lobrk;',		# ⟦ / &lobrk; / &LeftDoubleBracket;
    '\u21bc': '&lharu;',		# ↼ / &lharu; / &LeftVector; / &leftharpoonup;
    '\u21c7': '&llarr;',		# ⇇ / &llarr; / &leftleftarrows;
    '\u2194': '&harr;',		    # ↔ / &harr; / &leftrightarrow; / &LeftRightArrow;
    '\u21cb': '&lrhar;',		# ⇋ / &lrhar; / &leftrightharpoons; / &ReverseEquilibrium;
    '\u21a4': '&mapstoleft;',	# ↤ / &mapstoleft; / &LeftTeeArrow;
    '\u22cb': '&lthree;',		# ⋋ / &lthree; / &leftthreetimes;
    '\u22b2': '&vltri;',		# ⊲ / &vltri; / &LeftTriangle; / &vartriangleleft;
    '\u22b4': '&ltrie;',		# ⊴ / &ltrie; / &trianglelefteq; / &LeftTriangleEqual;
    '\u21bf': '&uharl;',		# ↿ / &uharl; / &LeftUpVector; / &upharpoonleft;
    '\u2308': '&lceil;',        # ⌈ / &lceil; / &LeftCeiling;
    '\u230a': '&lfloor;',       # ⌊ / &lfloor; / &LeftFloor;
    '\u2a8b': '&lEg;',		    # ⪋ / &lEg; / &lesseqqgtr;
    '\u22da': '&leg;',		    # ⋚ / &leg; / &lesseqgtr; / &LessEqualGreater;
    '\u2a7d': '&les;',		    # ⩽ / &les; / &leqslant; / &LessSlantEqual;
    '\u22d6': '&ltdot;',		# ⋖ / &ltdot; / &lessdot;
    '\u2276': '&lg;',		    # ≶ / &lg; / &lessgtr; / &LessGreater;
    '\u2272': '&lsim;',		    # ≲ / &lsim; / &lesssim; / &LessTilde;
    '\u226a': '&ll;',		    # ≪ / &ll; / &Lt; / &NestedLessLess;
    '\u23b0': '&lmoust;',		# ⎰ / &lmoust; / &lmoustache;
    '\u2a89': '&lnap;',		    # ⪉ / &lnap; / &lnapprox;
    '\u2268': '&lnE;',		    # ≨ / &lnE; / &lneqq;
    '\u2a87': '&lne;',		    # ⪇ / &lne; / &lneq;
    CHAR_ARROW_L: '&xlarr;',		# ⟵ / &xlarr; / &longleftarrow; / &LongLeftArrow;
    CHAR_ARROW_R: '&xrarr;',  # ⟶ / &xrarr; / &LongRightArrow; / &longrightarrow;
    CHAR_ARROW_LR: '&xharr;',		# ⟷ / &xharr; / &longleftrightarrow; / &LongLeftRightArrow;
    '\u27fc': '&xmap;',		    # ⟼ / &xmap; / &longmapsto;
    '\u21ac': '&rarrlp;',		# ↬ / &rarrlp; / &looparrowright;
    '\u201e': '&bdquo;',		# „ / &bdquo; / &ldquor;
    '\u2199': '&swarr;',		# ↙ / &swarr; / &swarrow; / &LowerLeftArrow;
    '\u2198': '&searr;',		# ↘ / &searr; / &searrow; / &LowerRightArrow;
    '\u21b0': '&lsh;',		    # ↰ / &Lsh; / &lsh;
    '\u25c3': '&ltri;',		    # ◃ / &ltri; / &triangleleft;
    '\u2720': '&malt;',		    # ✠ / &malt; / &maltese;
    '\u21a6': '&map;',		    # ↦ / &map; / &mapsto; / &RightTeeArrow;
    '\u21a5': '&mapstoup;',		# ↥ / &mapstoup; / &UpTeeArrow;
    '\u2133': '&Mscr;',		    # ℳ / &Mscr; / &phmmat; / &Mellintrf;
    '\u2223': '&mid;',		    # ∣ / &mid; / &smid; / &shortmid; / &VerticalBar;
    '\u2213': '&mp;',		    # ∓ / &mp; / &mnplus; / &MinusPlus;
    CHAR_HELLIP: '&mldr;',	    # … / &mldr; / &hellip;
    '\u22b8': '&mumap;',		# ⊸ / &mumap; / &multimap;
    '\u2249': '&nap;',		    # ≉ / &nap; / &napprox; / &NotTildeTilde;
    '\u266e': '&natur;',		# ♮ / &natur; / &natural;
    '\u2115': '&Nopf;',		    # ℕ / &Nopf; / &naturals;
    '\u2247': '&ncong;',		# ≇ / &ncong; / &NotTildeFullEqual;
    '\u2197': '&nearr;',		# ↗ / &nearr; / &nearrow; / &UpperRightArrow;
    '\u200b': '&ZeroWidthSpace;',		#   / &ZeroWidthSpace; / &NegativeThinSpace; / &NegativeMediumSpace;
                                        # &NegativeThickSpace; / &NegativeVeryThinSpace;
    '\u2262': '&nequiv;',		# ≢ / &nequiv; / &NotCongruent;
    '\u2928': '&toea;',		    # ⤨ / &toea; / &nesear;
    '\u2203': '&exist;',		# ∃ / &exist; / &Exists;
    '\u2204': '&nexist;',		# ∄ / &nexist; / &nexists; / &NotExists;
    '\u2271': '&nge;',		    # ≱ / &nge; / &ngeq; / &NotGreaterEqual;
    '\u2275': '&ngsim;',		# ≵ / &ngsim; / &NotGreaterTilde;
    '\u226f': '&ngt;',		    # ≯ / &ngt; / &ngtr; / &NotGreater;
    '\u21ce': '&nhArr;',		# ⇎ / &nhArr; / &nLeftrightarrow;
    '\u21ae': '&nharr;',		# ↮ / &nharr; / &nleftrightarrow;
    '\u220b': '&ni;',		    # ∋ / &ni; / &niv; / &SuchThat; / &ReverseElement;
    '\u21cd': '&nlArr;',		# ⇍ / &nlArr; / &nLeftarrow;
    '\u219a': '&nlarr;',		# ↚ / &nlarr; / &nleftarrow;
    '\u2270': '&nle;',		    # ≰ / &nle; / &nleq; / &NotLessEqual;
    '\u226e': '&nlt;',		    # ≮ / &nlt; / &nless; / &NotLess;
    '\u2274': '&nlsim;',		# ≴ / &nlsim; / &NotLessTilde;
    '\u22ea': '&nltri;',		# ⋪ / &nltri; / &ntriangleleft; / &NotLeftTriangle;
    '\u22ec': '&nltrie;',		# ⋬ / &nltrie; / &ntrianglelefteq; / &NotLeftTriangleEqual;
    '\u2224': '&nmid;',		    # ∤ / &nmid; / &nsmid; / &nshortmid; / &NotVerticalBar;
    '\u2226': '&npar;',		    # ∦ / &npar; / &nspar; / &nparallel; / &nshortparallel; / &NotDoubleVerticalBar;
    '\u2209': '&notin;',		# ∉ / &notin; / &notinva; / &NotElement;
    '\u2279': '&ntgl;',		    # ≹ / &ntgl; / &NotGreaterLess;
    '\u2278': '&ntlg;',		    # ≸ / &ntlg; / &NotLessGreater;
    '\u220c': '&notni;',		# ∌ / &notni; / &notniva; / &NotReverseElement;
    '\u2280': '&npr;',		    # ⊀ / &npr; / &nprec; / &NotPrecedes;
    '\u22e0': '&nprcue;',		# ⋠ / &nprcue; / &NotPrecedesSlantEqual;
    '\u22eb': '&nrtri;',		# ⋫ / &nrtri; / &ntriangleright; / &NotRightTriangle;
    '\u22ed': '&nrtrie;',		# ⋭ / &nrtrie; / &ntrianglerighteq; / &NotRightTriangleEqual;
    '\u22e2': '&nsqsube;',		# ⋢ / &nsqsube; / &NotSquareSubsetEqual;
    '\u22e3': '&nsqsupe;',		# ⋣ / &nsqsupe; / &NotSquareSupersetEqual;
    '\u2288': '&nsube;',		# ⊈ / &nsube; / &nsubseteq; / &NotSubsetEqual;
    '\u2281': '&nsc;',		    # ⊁ / &nsc; / &nsucc; / &NotSucceeds;
    '\u22e1': '&nsccue;',		# ⋡ / &nsccue; / &NotSucceedsSlantEqual;
    '\u2289': '&nsupe;',		# ⊉ / &nsupe; / &nsupseteq; / &NotSupersetEqual;
    '\u2241': '&nsim;',		    # ≁ / &nsim; / &NotTilde;
    '\u2244': '&nsime;',		# ≄ / &nsime; / &nsimeq; / &NotTildeEqual;
    '\u21cf': '&nrArr;',		# ⇏ / &nrArr; / &nRightarrow;
    '\u219b': '&nrarr;',		# ↛ / &nrarr; / &nrightarrow;
    '\u2196': '&nwarr;',		# ↖ / &nwarr; / &nwarrow; / &UpperLeftArrow;
    '\u2134': '&oscr;',		    # ℴ / &oscr; / &order; / &orderof;
    '\u203e': '&oline;',		#  ̄ / &oline; / &OverBar;
    '\u23b4': '&tbrk;',		    # ⎴ / &tbrk; / &OverBracket;
    '\u03d6': '&piv;',          # ϖ / &piv; / &varpi;
    '\u03d5': '&phiv;',		    # ϕ / &phiv; / &varphi; / &straightphi;
    '\u2665': '&hearts;',		# ♥ / &hearts; / &heartsuit; /
    '\u2119': '&Popf;',		    # ℙ / &Popf; / &primes;
    '\u227a': '&pr;',		    # ≺ / &pr; / &prec; / &Precedes;
    '\u2ab7': '&prap;',		    # ⪷ / &prap; / &precapprox;
    '\u227c': '&prcue;',		# ≼ / &prcue; / &preccurlyeq; / &PrecedesSlantEqual;
    '\u2aaf': '&pre;',		    # ⪯ / &pre; / &preceq; / &PrecedesEqual;
    '\u227e': '&prsim;',		# ≾ / &prsim; / &precsim; / &PrecedesTilde;
    '\u2ab9': '&prnap;',		# ⪹ / &prnap; / &precnapprox;
    '\u2ab5': '&prnE;',		    # ⪵ / &prnE; / &precneqq;
    '\u22e8': '&prnsim;',		# ⋨ / &prnsim; / &precnsim;
    '\u220f': '&prod;',         # ∏ / &prod; / &Product;
    '\u221d': '&prop;',		    # ∝ / &prop; / &vprop; / &propto; / &varpropto; / &Proportional;
    '\u211a': '&Qopf;',		    # ℚ / &Qopf; / &rationals;
    '\u21db': '&rAarr;',		# ⇛ / &rAarr; / &Rrightarrow;
    '\u27e9': '&rang;',		    # ⟩ / &rang; / &rangle; / &RightAngleBracket;
    '\u21a0': '&Rarr;',		    # ↠ / &Rarr; / &twoheadrightarrow;
    '\u21e5': '&rarrb;',		# ⇥ / &rarrb; / &RightArrowBar;
    '\u21a3': '&rarrtl;',		# ↣ / &rarrtl; / &rightarrowtail;
    '\u2309': '&rceil;',        # ⌉ / &rceil; / &RightCeiling;
    '\u219d': '&rarrw;',		# ↝ / &rarrw; / &rightsquigarrow;
    '\u03a9': '&ohm;',          # Ω / &ohm; / &Omega;
    '\u211c': '&Re;',		    # ℜ / &real; / &Re; / &Rfr; / &realpart;
    '\u211b': '&Rscr;',		    # ℛ / &Rscr; / &realine;
    '\u211d': '&Ropf;',		    # ℝ / &Ropf; / &reals;
    '\u21c0': '&rharu;',		# ⇀ / &rharu; / &RightVector; / &rightharpoonup;
    '\u03f1': '&rhov;',		    # ϱ / &rhov; / &varrho;
    '\u2192': '&rarr;',		    # → / &rarr; / &srarr; / &rightarrow; / &RightArrow; / &ShortRightArrow;
    '\u21c4': '&rlarr;',		# ⇄ / &rlarr; / &rightleftarrows; / &RightArrowLeftArrow;
    '\u27e7': '&robrk;',		# ⟧ / &robrk; / &RightDoubleBracket;
    '\u230b': '&rfloor;',       # ⌋ / &rfloor; / &RightFloor;
    '\u21c9': '&rrarr;',		# ⇉ / &rrarr; / &rightrightarrows;
    '\u22a2': '&vdash;',		# ⊢ / &vdash; / &RightTee;
    '\u22cc': '&rthree;',		# ⋌ / &rthree; / &rightthreetimes;
    '\u22b3': '&vrtri;',		# ⊳ / &vrtri; / &RightTriangle; / &vartriangleright;
    '\u22b5': '&rtrie;',		# ⊵ / &rtrie; / &trianglerighteq; / &RightTriangleEqual;
    '\u21be': '&uharr;',		# ↾ / &uharr; / &RightUpVector; / &upharpoonright;
    '\u23b1': '&rmoust;',		# ⎱ / &rmoust; / &rmoustache;
    '\u201c': '&ldquo;',        # “ / &ldquo; / &OpenCurlyDoubleQuote;
    '\u2018': '&lsquo;',        # ‘ / &lsquo; / &OpenCurlyQuote;
    '\u21b1': '&rsh;',		    # ↱ / &rsh; / &Rsh;
    '\u25b9': '&rtri;',		    # ▹ / &rtri; / &triangleright;
    '\u227b': '&sc;',		    # ≻ / &sc; / &succ; / &Succeeds;
    '\u2ab8': '&scap;',		    # ⪸ / &scap; / &succapprox;
    '\u227d': '&sccue;',		# ≽ / &sccue; / &succcurlyeq; / &SucceedsSlantEqual;
    '\u2ab0': '&sce;',		    # ⪰ / &sce; / &succeq; / &SucceedsEqual;
    '\u2aba': '&scnap;',		# ⪺ / &scnap; / &succnapprox;
    '\u2ab6': '&scnE;',		    # ⪶ / &scnE; / &succneqq;
    '\u22e9': '&scnsim;',		# ⋩ / &scnsim; / &succnsim;
    '\u227f': '&scsim;',		# ≿ / &scsim; / &succsim; / &SucceedsTilde;
    '\u2929': '&tosa;',		    # ⤩ / &tosa; / &seswar;
    '\u03c2': '&sigmaf;',		# ς / &sigmaf; / &sigmav; / &varsigma;
    '\u2243': '&sime;',		    # ≃ / &sime; / &simeq; / &TildeEqual;
    '\u2323': '&smile;',		# ⌣ / &smile; / &ssmile;
    '\u2660': '&spades;',		# ♠ / &spades; / &spadesuit; /
    '\u2293': '&sqcap;',		# ⊓ / &sqcap; / &SquareIntersection;
    '\u2294': '&sqcup;',		# ⊔ / &sqcup; / &SquareUnion;
    '\u221a': '&Sqrt;',		    # √ / &Sqrt; / &radic;
    '\u228f': '&sqsub;',		# ⊏ / &sqsub; / &sqsubset; / &SquareSubset;
    '\u2291': '&sqsube;',		# ⊑ / &sqsube; / &sqsubseteq; / &SquareSubsetEqual;
    '\u2290': '&sqsup;',		# ⊐ / &sqsup; / &sqsupset; / &SquareSuperset;
    '\u2292': '&sqsupe;',		# ⊒ / &sqsupe; / &sqsupseteq; / &SquareSupersetEqual;
    '\u25a1': '&squ;',		    # □ / &squ; / &Square; / &square;
    '\u22c6': '&Star;',		    # ⋆ / &Star; / &sstarf;
    '\u22d0': '&Sub;',		    # ⋐ / &Sub; / &Subset;
    '\u2282': '&sub;',		    # ⊂ / &sub; / &subset;
    '\u2ac5': '&subE;',		    # ⫅ / &subE; / &subseteqq;
    '\u2acb': '&subnE;',		# ⫋ / &subnE; / &subsetneqq;
    '\u228a': '&subne;',		# ⊊ / &subne; / &subsetneq;
    '\u2286': '&sube;',		    # ⊆ / &sube; / &subseteq; / &SubsetEqual;
    '\u2211': '&sum;',		    # ∑ / &sum; / &Sum;
    '\u22d1': '&Sup;',		    # ⋑ / &Sup; / &Supset;
    '\u2ac6': '&supE;',		    # ⫆ / &supE; / &supseteqq;
    '\u2283': '&sup;',		    # ⊃ / &sup; / &supset; / &Superset;
    '\u2287': '&supe;', 		# ⊇ / &supe; / &supseteq; / &SupersetEqual;
    '\u2acc': '&supnE;',		# ⫌ / &supnE; / &supsetneqq;
    '\u228b': '&supne;',		# ⊋ / &supne; / &supsetneq;
    '\u223c': '&sim;',          # ∼ / &sim; / &Tilde; / &thksim; / &thicksim;
    '\u2245': '&cong;',         # ≅ / &cong; / &TildeFullEqual;
    '\u20db': '&tdot;',		    # ⃛ / &tdot; / &TripleDot;
    '\u2234': '&there4;',		# ∴ / &there4; / &Therefore; / &therefore;
    '\u03d1': '&thetav;',		# ϑ / &thetav; / &vartheta; / &thetasym;
    CHAR_TRADE: '&trade;',		# ™ / &trade; / &TRADE;
    '\u25b5': '&utri;',		    # ▵ / &utri; / &triangle;
    '\u225c': '&trie;',		    # ≜ / &trie; / &triangleq;
    '\u21c5': '&udarr;',		# ⇅ / &udarr; / &UpArrowDownArrow;
    '\u296e': '&udhar;',		# ⥮ / &udhar; / &UpEquilibrium;
    '\u231c': '&ulcorn;',		# ⌜ / &ulcorn; / &ulcorner;
    '\u03d2': '&Upsi;',         # ϒ / &Upsi; / &upsih;
    '\u03c5': '&upsi;',		    # υ / &upsi; / &upsilon;
    '\u228e': '&uplus;',		# ⊎ / &uplus; / &UnionPlus;
    '\u2195': '&varr;',		    # ↕ / &varr; / &updownarrow; / &UpDownArrow;
    '\u2191': '&uarr;',         # ↑ / &uarr; / &uparrow; / &UpArrow; / &ShortUpArrow;
    '\u21c8': '&uuarr;',		# ⇈ / &uuarr; / &upuparrows;
    '\u231d': '&urcorn;',		# ⌝ / &urcorn; / &urcorner;
    '\u2016': '&Vert;',		    # ‖ / &Vert; / &Verbar;
    '\u2228': '&or;',		    # ∨ / &or; / &vee;
    CHAR_THIN_SP: '&thinsp;',	#   / &thinsp; / &ThinSpace;
    '\u2240': '&wr;',		    # ≀ / &wr; / &wreath; / &VerticalTilde;
    '\u2128': '&Zfr;',		    # ℨ / &Zfr; / &zeetrf;
    '\u2118': '&wp;',           # ℘ / &wp; / &weierp;
}

# === Динамическая генерация карт преобразования ===

def _build_translation_maps() -> dict[str, str]:
    """
    Создает карту для кодирования на лету, используя все доступные источники
    из html.entities и строгий порядок приоритетов для обеспечения
    предсказуемого и детерминированного результата.
    """
    # ШАГ 1: Создаем ЕДИНУЮ и ПОЛНУЮ карту {каноническое_имя: числовой_код}.
    # Это решает проблему разных форматов и дубликатов с точкой с запятой.
    unified_name2codepoint = {}

    # Сначала обрабатываем большой исторический словарь.
    for name, codepoint in entities.name2codepoint.items():
        # Нормализуем имя СРАЗУ, убирая опциональную точку с запятой (в html.entities предусмотрено, что иногда
        # символ `;` не ставится всякими неаккуратными верстальщиками и парсерами).
        canonical_name = name.rstrip(';')
        unified_name2codepoint[canonical_name] = codepoint
    # Затем обновляем его современным стандартом html5.
    # Это гарантирует, что если мнемоника есть в обоих, будет использована версия из html5.
    for name, char in entities.html5.items():
        # НОВОЕ: Проверяем, что значение является ОДИНОЧНЫМ символом.
        # Наш кодек, основанный на str.translate, не может обрабатывать
        # мнемоники, которые соответствуют строкам из нескольких символов
        # (например, символ + вариативный селектор). Мы их игнорируем.
        if len(char) != 1:
            continue
        # Нормализуем имя СРАЗУ.
        canonical_name = name.rstrip(';')
        unified_name2codepoint[canonical_name] = ord(char)

    # Теперь у нас есть полный и консистентный словарь unified_name2codepoint.
    # На его основе строим нашу карту для кодирования.
    encode_map = {}

    # ШАГ 2: Высший приоритет. Загружаем наши кастомные правила.
    encode_map.update(CUSTOM_ENCODE_MAP)

    # ШАГ 3: Следующий приоритет. Добавляем числовое кодирование.
    for char in ALWAYS_ENCODE_TO_NUMERIC_CHARS:
        if char not in encode_map:
            encode_map[char] = f'&#{ord(char)};'

    # ШАГ 4: Низший приоритет. Заполняем все остальное из нашей
    # объединенной и нормализованной карты unified_name2codepoint.
    for name, codepoint in unified_name2codepoint.items():
        char = chr(codepoint)
        if char not in encode_map and char not in NEVER_ENCODE_CHARS:
            # Теперь 'name' - это уже каноническое имя без ';',
            # поэтому дополнительная нормализация не нужна. Код стал проще!
            encode_map[char] = f'&{name};'

    return encode_map


# Создаем карту один раз при импорте модуля.
ENCODE_MAP = _build_translation_maps()

# --- Публичный API модуля ---
def get_encode_map():
    """Возвращает готовую карту для кодирования."""
    return ENCODE_MAP


# === КОНСТАНТЫ ДЛЯ ЕДИНИЦ ИЗМЕРЕНИЯ ===
# ТОЛЬКО АТОМАРНЫЕ единицы измерения: 'г', 'м', 'с', 'км', 'кв', 'куб', 'ч' и так далее.
# Никаких сложных и составных, типа: 'кв.м.', 'км/ч' или "до н.э." ...
# Пост-позиционные (можно ставить точку после, но не обязательно) (км, г., с.)
DEFAULT_POST_UNITS = [
    # Русские
    # --- Время и эпохи ---
    'гг', 'г.', 'в.', 'вв', 'н', 'э', 'сек', 'с.', 'мин', 'ч',
    # --- Масса и объём ---
    'кг', 'мг', 'ц', 'т', 'л', 'мл',
    # --- Размеры ---
    'кв', 'куб', 'мм', 'см', 'м', 'км', 'сот', 'га', 'м²', 'м³',
    # --- Финансы и количество ---
    'руб', 'коп', 'тыс', 'млн', 'млрд', 'трлн', 'трлрд', 'шт', 'об', 'ящ', 'уп', 'кор', 'пар', 'комп',
    # --- Издательское дело ---
    'пп', 'стр', 'рис', 'гр', 'табл', 'гл', 'п', 'пт', 'гл', 'том', 'т.', 'кн', 'илл', 'ред', 'изд', 'пер',
    # --- Физические и технические ---
    'дБ', 'Вт', 'кВт', 'МВт', 'ГВт', 'А', 'В', 'Ом', 'Па', 'кПа', 'МПа', 'Бар', 'кБар', 'Гц', 'кГц', 'МГц', 'ГГц',
    'рад', 'К', '°C', '°F', '%', 'мкм', 'нм', 'А°', 'эВ', 'Дж', 'кДж', 'МДж', 'пкФ', 'нФ', 'мкФ', 'мФ', 'Ф',
    'Гн', 'мГн', 'мкГн', 'Тл', 'Гс', 'эрг', 'бод', 'бит', 'байт', 'Кб', 'Мб', 'Гб', 'Тб', 'Пб', 'Эб', 'кал', 'ккал',
    # Английские
    # --- Издательское дело ---
    'pp', 'p', 'para', 'sect', 'fig', 'vol', 'ed', 'rev', 'dpi',
    # --- Имперские и американские единицы ---
    'in', 'ft', 'yd', 'mi', 'oz', 'lb', 'st', 'pt', 'qt', 'gal', 'mph', 'rpm', 'hp', 'psi', 'cal',
]
# Пред-позиционные (№ 5, $ 10)
DEFAULT_PRE_UNITS = ['№', '$', '€', '£', '₽', '#', '§', '¤', '₴', '₿', '₺', '₦', '₩', '₪', '₫', '₲', '₡', '₵',
                     'ГОСТ', 'ТУ', 'ИСО', 'DIN', 'ASTM', 'EN', 'IEC', 'IEEE'] # технические стандарты перед числом работают как единицы измерения

# Операторы, которые могут стоять между единицами измерения (км/ч)
# Сложение и вычитание здесь намеренно отсутствуют.
UNIT_MATH_OPERATORS = ['/', '*', '×', CHAR_MIDDOT, '÷']

# === КОНСТАНТЫ ДЛЯ ФИНАЛЬНЫХ СОКРАЩЕНИЙ ===
# Эти сокращения (обычно в конце фразы) будут "склеены" тонкой шпацией, а перед ними будет поставлен неразрывный пробел.
# Важно, чтобы многосложные сокращения (типа "и т. д.") были в списке с разделителем пробелом (иначе мы не сможем их найти).
ABBR_COMMON_FINAL = [
     # 'т. д.', 'т. п.', 'др.', 'пр.',
     # УБРАНЫ из-за неоднозначности: др. -- "другой", "доктор", "драм" / пр. -- "прочие", "профессор", "проект", "проезд" ...
    'т. д.', 'т. п.',
]

ABBR_COMMON_PREPOSITION = [
    'т. е.', 'т. к.', 'т. о.', 'т. ч.',
    'и. о.', 'ио', 'вр. и. о.', 'врио',
    'тов.', 'г-н.', 'г-жа.', 'им.',
    'д. о. с.', 'д. о. н.', 'д. м. н.', 'к. т. д.', 'к. т. п.',
    'АО', 'ООО', 'ЗАО', 'ПАО', 'НКО', 'ОАО', 'ФГУП', 'НИИ', 'ПБОЮЛ', 'ИП',
]

# === КОНСТАНТЫ ДЛЯ HTML-ТЕГОВ, ВНУТРИ КОТОРЫХ НЕ НАДО ТИПОГРАФИРОВАТЬ ===
PROTECTED_HTML_TAGS = ['style', 'script', 'pre', 'code', 'kbd', 'samp', 'math']

# === КОНСТАНТЫ ДЛЯ ВИСЯЧЕЙ ТИПОГРАФИКИ ===

# 1. Набор символов, которые могут "висеть" слева
HANGING_PUNCTUATION_LEFT_CHARS = frozenset([
    CHAR_RU_QUOT1_OPEN,   # «
    CHAR_EN_QUOT1_OPEN,   # “
    '(', '[', '{',
])

# 2. Набор символов, которые могут "висеть" справа
HANGING_PUNCTUATION_RIGHT_CHARS = frozenset([
    CHAR_RU_QUOT1_CLOSE,  # »
    CHAR_EN_QUOT1_CLOSE,  # ”
    ')', ']', '}',
    '.', ',', ':',
])

# 3. Словарь, сопоставляющий символ с его CSS-классом
HANGING_PUNCTUATION_CLASSES = {
    # Левая пунктуация: все классы начинаются с 'etp-l'
    CHAR_RU_QUOT1_OPEN: 'etp-laquo',
    CHAR_EN_QUOT1_OPEN: 'etp-ldquo',
    '(': 'etp-lpar',
    '[': 'etp-lsqb',
    '{': 'etp-lcub',
    # Правая пунктуация: все классы начинаются с 'etp-r'
    CHAR_RU_QUOT1_CLOSE: 'etp-raquo',
    CHAR_EN_QUOT1_CLOSE: 'etp-rdquo',
    ')': 'etp-rpar',
    ']': 'etp-rsqb',
    '}': 'etp-rcub',
    '.': 'etp-r-dot',
    ',': 'etp-r-comma',
    ':': 'etp-r-colon',
}


# tests/test _layout.py
# Тестирует модуль LayoutProcessor. Проверяет обработку тире и специальных символов в тексте.

import pytest
from etpgrf.layout import LayoutProcessor, CHAR_THIN_SP
from etpgrf.config import CHAR_NBSP, CHAR_HELLIP, CHAR_THIN_SP, CHAR_UNIT_SEPARATOR

LAYOUT_TEST_CASES = [
    # --- Длинное тире (—) для русского языка ---
    ('ru', 'Слово — слово', f'Слово{CHAR_NBSP}— слово'),
    ('ru', 'В начале — слово', f'В начале{CHAR_NBSP}— слово'),
    ('ru', 'Слово — в конце.', f'Слово{CHAR_NBSP}— в конце.'),
    ('ru-en', 'Слово — слово', f'Слово{CHAR_NBSP}— слово'),  # Приоритет у 'ru'

    # --- Длинное тире (—) для английского языка ---
    ('en', 'Word — word', 'Word—word'),
    ('en', 'Start — word', 'Start—word'),
    ('en', 'Word — end.', 'Word—end.'),
    ('en-ru', 'Word — word', 'Word—word'),  # Приоритет у 'en'

    # --- Среднее тире (–) также должно обрабатываться ---
    ('ru', 'Слово – слово', f'Слово{CHAR_NBSP}– слово'),
    ('en', 'Word – word', 'Word–word'),

# --- Случаи тире рядом с пунктуацией и кавычками ---
    ('ru', 'Да, — сказал он', f'Да,{CHAR_NBSP}— сказал он'),
    ('en', 'Yes, — he said', 'Yes,—he said'),
    ('ru', '«Слово», — сказал он', f'«Слово»,{CHAR_NBSP}— сказал он'),
    ('en', '“Word,” — he said', '“Word,”—he said'),
    ('ru', 'Слово! — воскликнул он.', f'Слово!{CHAR_NBSP}— воскликнул он.'),
    ('en', 'Word! — he exclaimed.', 'Word!—he exclaimed.'),
    # Тире после закрывающей кавычки
    ('ru', '«Слово» — это важно.', f'«Слово»{CHAR_NBSP}— это важно.'),
    ('en', '“Word” — is important.', '“Word”—is important.'),

    # --- Случаи, которые не должны меняться ---
    ('ru', 'слово—слово', 'слово—слово'),  # Уже слитно, не трогаем
    ('en', 'word—word', 'word—word'),  # Уже слитно, не трогаем
    ('ru', "что-нибудь такое", "что-нибудь такое"),  # Дефис, а не минус
    ('ru', "что-нибудь такое", "что-нибудь такое"),  # Минус
    ('ru', "что-\nнибудь такое", "что-\nнибудь такое"),  # Минус
    ('ru', ' — слово', ' — слово'),  # Пробел в начале строки, не трогаем
    ('en', ' — word', ' — word'),  # Пробел в начале строки, не трогаем
    ('ru', 'слово — ', 'слово — '),  # Пробел в конце строки, не трогаем
    ('en', 'word — ', 'word — '),  # Пробел в конце строки, не трогаем
    ('ru', '1941–1945', '1941–1945'),  # Диапазон без пробелов (через короткое тире) не трогаем
    ('ru', '1941—1945', '1941—1945'),  # Диапазон (через длинное тире) не трогаем
    ('ru', '1941-1945', '1941-1945'),  # Диапазон (через дефис/минус) не трогаем (будет преобразован в SymbolsProcessor)
    ('ru', '1 — 2', '1 — 2'),  # Диапазон с пробелами (цифры!) не трогаем
    ('ru', '1 - 2', '1 - 2'),  # Математика (минус) не трогаем

    # --- Многоточие ---
    ('ru', f"Что это{CHAR_HELLIP} \n \t не знаю.", f"Что это{CHAR_HELLIP}{CHAR_NBSP}не знаю."),
    ('ru', f"Что это{CHAR_HELLIP} не знаю.", f"Что это{CHAR_HELLIP}{CHAR_NBSP}не знаю."),
    ('ru', f"Что это{CHAR_HELLIP} 123.", f"Что это{CHAR_HELLIP}{CHAR_NBSP}123."),
    (f'ru', f"Что это{CHAR_HELLIP}", f"Что это{CHAR_HELLIP}"),  # Не меняется в конце
    (f'ru', f"Что это{CHAR_HELLIP}   ", f"Что это{CHAR_HELLIP}   "),  # Не меняется в конце
    (f'ru', f"1{CHAR_HELLIP}2{CHAR_HELLIP}3{CHAR_HELLIP}4{CHAR_HELLIP}5, я иду тебя искать!",
            f"1{CHAR_HELLIP}2{CHAR_HELLIP}3{CHAR_HELLIP}4{CHAR_HELLIP}5, я иду тебя искать!"),
    (f'ru', f"1{CHAR_HELLIP}2{CHAR_HELLIP}3{CHAR_HELLIP}4{CHAR_HELLIP}5{CHAR_HELLIP} я иду тебя искать!",
            f"1{CHAR_HELLIP}2{CHAR_HELLIP}3{CHAR_HELLIP}4{CHAR_HELLIP}5{CHAR_HELLIP}{CHAR_NBSP}я иду тебя искать!"),

    # --- Отрицательные числа ---
    ('ru', "температура -10 градусов", f"температура{CHAR_NBSP}-10 градусов"),
    ('ru', "от -5 до +5", f"от{CHAR_NBSP}-5 до +5"),
    ('ru', "в диапазоне ( -10, 10)", f"в диапазоне ({CHAR_NBSP}-10, 10)"), # Пробел после скобки

    # --- Случаи, которые не должны меняться (отрицательные числа) ---
    ('ru', "10 - 5 = 5", "10 - 5 = 5"),  # Бинарный минус не трогаем
    ('ru', "слово-10", "слово-10"), # Дефис, а не минус
    ('ru', "1-2-3-4-5, я иду тебя искать", "1-2-3-4-5, я иду тебя искать"), # Дефис, а не минус
    ('ru', "в диапазоне (-10, 10)", f"в диапазоне (-10, 10)"),  # Без пробела после скобки

    # --- Инициалы (должны обрабатываться по умолчанию) ---
    # Разные комбинации пробелов
    ('ru', "А. С. Пушкин", f"А.{CHAR_NBSP}С.{CHAR_NBSP}Пушкин"),
    ('ru', "А.С. Пушкин", f"А.{CHAR_THIN_SP}С.{CHAR_NBSP}Пушкин"),
    ('ru', "А.С.Пушкин", f"А.{CHAR_THIN_SP}С.{CHAR_THIN_SP}Пушкин"),
    ('en', "J. R. R. Tolkien", f"J.{CHAR_NBSP}R.{CHAR_NBSP}R.{CHAR_NBSP}Tolkien"),
    ('en', "J.R.R. Tolkien", f"J.{CHAR_THIN_SP}R.{CHAR_THIN_SP}R.{CHAR_NBSP}Tolkien"),
    ('ru', "Пушкин А. С.", f"Пушкин{CHAR_NBSP}А.{CHAR_NBSP}С."),
    ('ru', "Пушкин А.С.", f"Пушкин{CHAR_NBSP}А.{CHAR_THIN_SP}С."),
    ('en', "Tolkien J. R. R.", f"Tolkien{CHAR_NBSP}J.{CHAR_NBSP}R.{CHAR_NBSP}R."),
    ('en', "Tolkien J.R.R.", f"Tolkien{CHAR_NBSP}J.{CHAR_THIN_SP}R.{CHAR_THIN_SP}R."),
    # Один инициал
    ('ru', "Это был В. Высоцкий.", f"Это был В.{CHAR_NBSP}Высоцкий."),
    ('ru', "Высоцкий В. был гением.", f"Высоцкий{CHAR_NBSP}В. был гением."),
    # Акронимы (бонус)
    ('ru', "Сделано в С.Ш.А.", f"Сделано в С.{CHAR_THIN_SP}Ш.{CHAR_THIN_SP}А."),
    ('ru', "Сделано в С. Ш. А.", f"Сделано в С.{CHAR_NBSP}Ш.{CHAR_NBSP}А."),
    ('en', "На замке стояло клеймо «Made in U. S. A.»",  f"На замке стояло клеймо «Made in U.{CHAR_NBSP}S.{CHAR_NBSP}A.»"),
    # Никаких изменений, если пробелы другого типа
    ('ru', "А.\u200DС.\u200AПушкин", "А.\u200DС.\u200AПушкин"),
    ('ru', "Пушкин А.\u200AС.", f"Пушкин{CHAR_NBSP}А.\u200AС."),
    ('en', "J.\u200DR.\u200DR.\u200ATolkien", "J.\u200DR.\u200DR.\u200ATolkien"),
    ('en', "Tolkien J.\u200AR.\u200AR.", f"Tolkien{CHAR_NBSP}J.\u200AR.\u200AR."),

    # --- Единицы измерения (по умолчанию) ---
    ('ru', "Радиус Солнца — около 696.340 км", f"Радиус Солнца{CHAR_NBSP}— около 696.340{CHAR_NBSP}км"),
    ('ru', "5 кг.", f"5{CHAR_NBSP}кг."),
    ('ru', "Доработки проекта стоили 100 тыс. руб.", f"Доработки проекта стоили 100{CHAR_NBSP}тыс.{CHAR_THIN_SP}руб."),
    ('ru', "№ 5", f"№{CHAR_NBSP}5"),
    ('ru', "Договор № 504/2025А", f"Договор №{CHAR_NBSP}504/2025А"),
    ('ru+en', "Доплата за багаж $ 45.50", f"Доплата за багаж ${CHAR_NBSP}45.50"),
    ('ru+en', "Инвестиции составили $2.5 млн.", f"Инвестиции составили $2.5{CHAR_NBSP}млн."),
    ('ru+en', "Инвестиции составили $ 2.5 млн.", f"Инвестиции составили ${CHAR_NBSP}2.5{CHAR_NBSP}млн."),
    ('ru', "За окном -5 °C", f"За окном{CHAR_NBSP}-5{CHAR_NBSP}°C"),
    # Сложные единицы (склеиваются тонкой шпацией, привязываются к числу неразрывным пробелом)
    # ('ru', "Дом 120 кв.м. / Участок 6 сот.", f"Дом 120{CHAR_NBSP}кв.м. / Участок 6{CHAR_NBSP}сот."),
    # ('ru', "Гробик кладут в ямку 2 кв. м.", f"Гробик кладут в ямку 2 кв. м."),
    ('ru', "IV-X вв.", f"IV-X{CHAR_NBSP}вв."),
    ('ru', "IV в. н. э.", f"IV{CHAR_NBSP}в.{CHAR_THIN_SP}н.{CHAR_THIN_SP}э."),
    ('ru+en', "Хаммурапи (1792 - 1750 до н. э.)",
             f"Хаммурапи (1792 - 1750 до н.{CHAR_THIN_SP}э.)"),

    # Составные и математические единицы
    ('ru', "Площадь 120 кв. м.", f"Площадь 120{CHAR_NBSP}кв.{CHAR_THIN_SP}м."),
    ('ru', "Площадь 130 кв.м.", f"Площадь 130{CHAR_NBSP}кв.{CHAR_THIN_SP}м."),
    ('ru', "Площадь 130 м²", f"Площадь 130{CHAR_NBSP}м²"),
    ('ru', "Площадь 130м²", f"Площадь 130м²"),
    ('ru', f"Площадь 140  {CHAR_NBSP} кв.{CHAR_NBSP}м.", f"Площадь 140{CHAR_NBSP}кв.{CHAR_THIN_SP}м."),
    ('ru', "Площадь 150 тыс. кв. км.", f"Площадь 150{CHAR_NBSP}тыс.{CHAR_THIN_SP}кв.{CHAR_THIN_SP}км."),
    ('ru', "Скорость 90 км/ч", f"Скорость 90{CHAR_NBSP}км/ч"),
    ('ru', "Скорость 90 км / ч", f"Скорость 90{CHAR_NBSP}км/ч"),
    ('ru', "В 500 г. н. э.", f"В 500{CHAR_NBSP}г.{CHAR_THIN_SP}н.{CHAR_THIN_SP}э."),
    ('ru', "Пластинка 45 мин. об.", f"Пластинка 45{CHAR_NBSP}мин.{CHAR_THIN_SP}об."),
    ('ru', "Пластинка 45 об. мин.", f"Пластинка 45{CHAR_NBSP}об.{CHAR_THIN_SP}мин."),
    ('ru', "За окном 15 °C", f"За окном 15{CHAR_NBSP}°C"),
    ('ru', "HiFi 20 Гц - 20 кГц", f"HiFi 20{CHAR_NBSP}Гц - 20{CHAR_NBSP}кГц"),
    # Случаи когда единица измерения вплотную к числу и не должны меняться
    ('ru', "Площадь 130м²", f"Площадь 130м²"),
    ('ru', "За окном 15°C", f"За окном 15°C"),
    ('ru', "Скорость 90км/ч", f"Скорость 90км/ч"),
    ('ru', "Зачислено на счёт $5тыс.", f"Зачислено на счёт $5тыс."),
    ('ru', "Зачислено на счёт $5000", f"Зачислено на счёт $5000"),

    # Сложные единицы (склеиваются тонкой шпацией, привязываются к числу неразрывным пробелом)
    ('ru', "Зачислено на счёт 10 млн.руб.", f"Зачислено на счёт 10{CHAR_NBSP}млн.{CHAR_THIN_SP}руб."),
    ('ru', "Дом 120 кв.м. / Участок 6 сот.", f"Дом 120{CHAR_NBSP}кв.{CHAR_THIN_SP}м. / Участок 6{CHAR_NBSP}сот."),
    # ('ru', "Гробик кладут в ямку 2 кв. м.", f"Гробик кладут в ямку 2 кв. м."),
    ('ru', "500 до н. э.", f"500 до н.{CHAR_THIN_SP}э."),
    ('ru+en', "Хаммурапи (1792 - 1750 до н. э.)", f"Хаммурапи (1792 - 1750 до н.{CHAR_THIN_SP}э.)"),

    # Финальные сокращения
    ('ru', "1 и т.д.", f"1 и{CHAR_NBSP}т.{CHAR_THIN_SP}д."),
    ('ru', "2 и т.   \n д.", f"2 и{CHAR_NBSP}т.{CHAR_THIN_SP}д."),
    ('ru', "3 и т.д., и др.", f"3 и{CHAR_NBSP}т.{CHAR_THIN_SP}д., и др."), # Слитное написание
    ('ru', "4 и т.п., и пр.", f"4 и{CHAR_NBSP}т.{CHAR_THIN_SP}п., и пр."), # Слитное написание
    ('ru', "5 и т. п., и т.п., и пр.", f"5 и{CHAR_NBSP}т.{CHAR_THIN_SP}п., и{CHAR_NBSP}т.{CHAR_THIN_SP}п., и пр."), # Слитное и раздельное написание

    # Препозиционные сокращения
    ('ru', "Назначить и.о. директора", f"Назначить и.{CHAR_THIN_SP}о.{CHAR_NBSP}директора"),
    ('ru', "Назначить ио директора", f"Назначить ио{CHAR_NBSP}директора"),  # без точек
    ('ru', "замечаний не было, т. е. не было и ошибок", f"замечаний не было, т.{CHAR_THIN_SP}е.{CHAR_NBSP}не было и ошибок"),
    ('ru', "Назначить и. о. директора", f"Назначить и.{CHAR_THIN_SP}о.{CHAR_NBSP}директора"),  # с пробелом
    ('ru', "Назначить и.о. директора", f"Назначить и.{CHAR_THIN_SP}о.{CHAR_NBSP}директора"),  # без пробела
    ('ru', "Назначить ио директора", f"Назначить ио{CHAR_NBSP}директора"),  # без точек
    ('ru', "то есть т. е. сказать", f"то есть т.{CHAR_THIN_SP}е.{CHAR_NBSP}сказать"),
    ('ru', "таким образом, т. о. мы видим", f"таким образом, т.{CHAR_THIN_SP}о.{CHAR_NBSP}мы видим"),
    ('ru', "потому что т.к. это важно", f"потому что т.{CHAR_THIN_SP}к.{CHAR_NBSP}это важно"),
    ('ru', "Назначить вр. и. о. начальника", f"Назначить вр.{CHAR_THIN_SP}и.{CHAR_THIN_SP}о.{CHAR_NBSP}начальника"),
    ('ru', "Назначить врио начальника", f"Назначить врио{CHAR_NBSP}начальника"),
    ('ru', "Выступает тов. Сухов", f"Выступает тов.{CHAR_NBSP}Сухов"),
    ('ru', "Приехал г-н. Петров", f"Приехал г-н.{CHAR_NBSP}Петров"),
    ('ru', "Институт им. Курчатова", f"Институт им.{CHAR_NBSP}Курчатова"),
    ('ru', "собаку оперировал д. м. н. профессор Преображенский", f"собаку оперировал д.{CHAR_THIN_SP}м.{CHAR_THIN_SP}н.{CHAR_NBSP}профессор Преображенский"),
    ('ru', "АО \"Рога и Копыта\"", f"АО{CHAR_NBSP}\"Рога и Копыта\""),

    # --- Комбинированные случаи ---
    ('ru', f"Да — это так{CHAR_HELLIP} а может и нет. Счёт -10.",
           f"Да{CHAR_NBSP}— это так{CHAR_HELLIP}{CHAR_NBSP}а может и нет. Счёт{CHAR_NBSP}-10."),
    ('ru', f"По мнению А.С.Пушкина — это...", f"По мнению А.{CHAR_THIN_SP}С.{CHAR_THIN_SP}Пушкина{CHAR_NBSP}— это..."),
    ('ru', f"К моменту смерти (1837 г.) А.С. Пушкин был должен 135.833 руб.: 92.500 руб. частным лицам и 43.333 руб."
           f" царской казне.",
           f"К моменту смерти (1837{CHAR_NBSP}г.) А.{CHAR_THIN_SP}С.{CHAR_NBSP}Пушкин был должен"
           f" 135.833{CHAR_NBSP}руб.: 92.500{CHAR_NBSP}руб. частным лицам и 43.333{CHAR_NBSP}руб."
           f" царской казне."),

]


@pytest.mark.parametrize("lang, input_string, expected_output", LAYOUT_TEST_CASES
)
def test_layout_processor_default(lang, input_string, expected_output):
    """Проверяет работу LayoutProcessor в изоляции."""
    processor = LayoutProcessor(langs=lang)
    actual_output = processor.process(input_string)
    assert actual_output == expected_output


# Тесты для проверки работы с кастомными опциями
LAYOUT_OPTIONS_TEST_CASES = [
    # --- Отключение правил ---
    # Отключение обработки единиц измерения
    ('ru', "10 км", "10 км", {'process_units': False}),
    ('ru', "№ 5", "№ 5", {'process_units': False}),
    ('ru', "100 тыс. руб.", "100 тыс. руб.", {'process_units': False}),
    # Отключение обработки инициалов
    ('ru', "А. С. Пушкин", "А. С. Пушкин", {'process_initials_and_acronyms': False}),
    ('ru', "Пушкин А.С.", "Пушкин А.С.", {'process_initials_and_acronyms': False}),

    # --- Кастомные единицы измерения ---
    # Кастомные единицы (список)
    ('ru', "100 тонн", f"100{CHAR_NBSP}тонн", {'process_units': ['тонн']}),
    # Кастомные единицы (строка)
    ('ru', "50 штук", f"50{CHAR_NBSP}штук", {'process_units': 'штук кг'}),
    # Стандартные единицы должны продолжать работать вместе с кастомными
    ('ru', "100 тонн и 10 км", f"100{CHAR_NBSP}тонн и 10{CHAR_NBSP}км", {'process_units': ['тонн']}),
    # Кастомные единицы не должны мешать другим правилам
    ('ru', "А.С. Пушкин получил 10 бочек селёдки",
           f"А.{CHAR_THIN_SP}С.{CHAR_NBSP}Пушкин получил 10{CHAR_NBSP}бочек селёдки", {'process_units': ['бочек']}),

    # --- Проверка безопасности ---
    # "Вредоносная" единица с сепаратором должна быть проигнорирована, а безопасная - обработана.
    ('ru', "10 вредных и 20 полезных", f"10 вредных и 20{CHAR_NBSP}полезных", {'process_units': [f'вредных{CHAR_UNIT_SEPARATOR}', 'полезных']}),
]


@pytest.mark.parametrize("lang, input_string, expected_output, options", LAYOUT_OPTIONS_TEST_CASES)
def test_layout_processor_with_options(lang, input_string, expected_output, options):
    """Проверяет работу LayoutProcessor с кастомными настройками."""
    processor = LayoutProcessor(langs=lang, **options)
    actual_output = processor.process(input_string)
    assert actual_output == expected_output


# etpgrf/codec.py
# Модуль для преобразования текста между Unicode и HTML-мнемониками.

import regex
import html
from . import config
# from etpgrf.config import (ALL_ENTITIES, ALWAYS_MNEMONIC_IN_SAFE_MODE, MODE_MNEMONIC, MODE_MIXED)

# --- Создаем словарь для кодирования Unicode -> Mnemonic ---
# Получаем готовую карту для кодирования один раз при импорте
_ENCODE_MAP = config.get_encode_map()
# Создаем таблицу для быстрой замены через str.translate
_TRANSLATE_TABLE = str.maketrans(_ENCODE_MAP)

#
# for name, (uni_char, mnemonic) in ALL_ENTITIES.items():
#     _ENCODE_MAP[uni_char] = mnemonic

# --- Основные функции кодека ---

def decode_to_unicode(text: str) -> str:
    """
    Преобразует все известные HTML-мнемоники и числовые коды в их
    Unicode-эквиваленты, используя стандартную библиотеку html.
    """
    if not text or '&' not in text:
        return text
    return html.unescape(text)


def encode_from_unicode(text: str, mode: str) -> str:
    """
    Преобразует Unicode-символы в HTML-мнемоники в соответствии с режимом.
    """
    if not text:
        # Если текст пустой, просто возвращаем его
        return text
    if mode == config.MODE_UNICODE:
        # В режиме 'unicode' ничего не делаем
        return text

    if mode == config.MODE_MNEMONIC:
        # В режиме 'mnemonic' заменяем все известные символы, используя
        # заранее скомпилированную таблицу для максимальной производительности.
        return text.translate(_TRANSLATE_TABLE)
    if mode == config.MODE_MIXED:
        # Создаем временную карту только для "безопасных" символов
        safe_map = {
            char: _ENCODE_MAP[char]
            for char in config.SAFE_MODE_CHARS_TO_MNEMONIC
            if char in _ENCODE_MAP
        }
        if not safe_map:
            return text
        return text.translate(str.maketrans(safe_map))

    # Возвращаем исходный текст, если режим не распознан
    return text



# etpgrf/logging_settings.py
import logging
from etpgrf.defaults import etpgrf_settings             # Импортируем наш объект настроек по умолчанию

# --- Корневой логгер для всей библиотеки etpgrf ---
# Имя логгера "etpgrf" позволит пользователям настраивать
# логирование для всех частей библиотеки.
# Например, logging.getLogger("etpgrf").setLevel(logging.DEBUG)
# или logging.getLogger("etpgrf.hyphenation").setLevel(logging.INFO)
_etpgrf_init_logger = logging.getLogger("etpgrf")


# --- Настройка корневого логгера ---
def setup_library_logging():
    """
    Настраивает корневой логгер для библиотеки etpgrf.
    Эту функцию следует вызывать один раз (например, при импорте
    основного модуля библиотеки или при первом обращении к логгеру).
    """
    # Проверяем инициализацию хандлеров логера, чтобы случайно не добавлять хендлеры многократно
    if not _etpgrf_init_logger.hasHandlers():
        log_level_to_set = logging.WARNING      # Значение по умолчанию
        # самый мощный формат, который мы можем использовать
        log_format_to_set = '%(asctime)s - %(name)s - %(levelname)s - %(module)s.%(funcName)s:%(lineno)d - %(message)s'
        # обычно достаточно:
        # log_format_to_set = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'      # Формат по умолчанию

        fin_message: str | None = None
        if hasattr(etpgrf_settings, 'logging_settings'):
            if hasattr(etpgrf_settings.logging_settings, 'LEVEL'):
                log_level_to_set = etpgrf_settings.logging_settings.LEVEL
            if hasattr(etpgrf_settings.logging_settings, 'FORMAT') and etpgrf_settings.logging_settings.FORMAT:
                log_format_to_set = etpgrf_settings.logging_settings.FORMAT
        else:
            # Этого не должно происходить, если defaults.py настроен правильно
            fin_message= "ПРЕДУПРЕЖДЕНИЕ: etpgrf_settings.logging_settings не найдены при начальной настройке логгера."

        _etpgrf_init_logger.setLevel(log_level_to_set)      # Устанавливаем уровень логирования
        console_handler = logging.StreamHandler()           # Создаем хендлер вывода в консоль
        console_handler.setLevel(log_level_to_set)          # Уровень для хендлера тоже
        formatter = logging.Formatter(log_format_to_set)    # Создаем форматтер для вывода
        console_handler.setFormatter(formatter)             # Устанавливаем форматтер для хендлера
        _etpgrf_init_logger.addHandler(console_handler)     # Добавляем хендлер в логгер
        if fin_message is not None:
            # Если есть сообщение об отсутствии настроек в `etpgrf_settings`, выводим его
            _etpgrf_init_logger.warning(fin_message)
        _etpgrf_init_logger.debug(f"Корневой логгер `etpgrf` инициализирован."
                                  f" Уровень: {logging.getLevelName(_etpgrf_init_logger.getEffectiveLevel())}")


# --- Динамическое изменение уровня логирования ---
def update_etpgrf_log_level_from_settings():
    """
    Обновляет уровень логирования для корневого логгера `etpgrf` и его
    обработчиков, читая значение из `etpgrf_settings.logging_settings.LEVEL`.
    """
    # Проверяем, что настройки логирования и уровень существуют в `defaults.etpgrf_settings`
    if not hasattr(etpgrf_settings, 'logging_settings') or \
       not hasattr(etpgrf_settings.logging_settings, 'LEVEL'):
        _etpgrf_init_logger.warning("Невозможно обновить уровень логгера: `etpgrf_settings.logging_settings.LEVEL`"
                                    " не найден.")
        return

    new_level = etpgrf_settings.logging_settings.LEVEL
    _etpgrf_init_logger.setLevel(new_level)
    for handler in _etpgrf_init_logger.handlers:
        handler.setLevel(new_level)     # Устанавливаем уровень для каждого хендлера

    _etpgrf_init_logger.info(f"Уровень логирования `etpgrf` динамически обновлен на:"
                             f" {logging.getLevelName(_etpgrf_init_logger.getEffectiveLevel())}")


# --- Динамическое изменение формата логирования ---
def update_etpgrf_log_format_from_settings():
    """
    Обновляет формат логирования для обработчиков корневого логгера etpgrf,
    читая значение из etpgrf_settings.logging_settings.FORMAT.
    """
    if not hasattr(etpgrf_settings, 'logging_settings') or \
       not hasattr(etpgrf_settings.logging_settings, 'FORMAT') or \
       not etpgrf_settings.logging_settings.FORMAT:
        _etpgrf_init_logger.warning("Невозможно обновить формат логгера: `etpgrf_settings.logging_settings.FORMAT`"
                                    " не найден или пуст.")
        return

    new_format_string = etpgrf_settings.logging_settings.FORMAT
    new_formatter = logging.Formatter(new_format_string)

    for handler in _etpgrf_init_logger.handlers:
        handler.setFormatter(new_formatter) # Применяем новый форматтер к каждому хендлеру

    _etpgrf_init_logger.info(f"Формат логирования для `etpgrf` динамически обновлен на: `{new_format_string}`")


# --- Инициализация логгера при первом импорте ---
setup_library_logging()


# --- Предоставление логгеров для модулей ---
def get_logger(name: str) -> logging.Logger:
    """
    Возвращает логгер для указанного имени.
    Обычно используется как logging.getLogger(__name__) в модулях.
    Имя будет дочерним по отношению к "etpgrf", например, "etpgrf.hyphenation".
    """
    # Убедимся, что имя логгера начинается с "etpgrf." для правильной иерархии,
    # если только это не сам корневой логгер.
    if not name.startswith("etpgrf") and name != "etpgrf":
         # Это может быть __name__ из модуля верхнего уровня, использующего библиотеку. В этом случае мы не хотим
         # делать его дочерним от "etpgrf" насильно. Просто вернем логгер с именем...
         # Либо можно настроить, что все логгеры, получаемые через эту функцию, должны быть частью иерархии "etpgrf"...
         # Для простоты оставим так:
         pass       # logging_settings = logging.getLogger(name)
    # Более правильный подход для модулей ВНУТРИ библиотеки etpgrf: они должны вызывать `logging.getLogger(__name__)`
    # напрямую. Тогда эта функция `get_logger()` может быть и не нужна, если модули ничего не делают кроме:
    #   import logging
    #   logging_settings = logging.getLogger(__name__)
    #
    # Однако, если нужно централизованно получать логгеры, можно сделать, чтобы `get_logger()` всегда возвращал
    # дочерний логгер:
    #   if not name.startswith("etpgrf."):
    #        name = f"etpgrf.{name}"
    return logging.getLogger(name)




# tests/test_sanitizer.py
# Тестирует модуль SanitizerProcessor.

import pytest
from bs4 import BeautifulSoup
from etpgrf.sanitizer import SanitizerProcessor
from etpgrf.config import SANITIZE_NONE, SANITIZE_ETPGRF, SANITIZE_ALL_HTML


def test_sanitizer_mode_none():
    """
    Проверяет, что в режиме SANITIZE_NONE (по умолчанию) ничего не происходит.
    """
    html_input = '<p><span class="etp-laquo">«</span>Hello<span class="user-class"> world</span>.</p>'
    soup = BeautifulSoup(html_input, 'html.parser')
    
    # Тестируем с mode=None и mode=False
    processor_none = SanitizerProcessor(mode=SANITIZE_NONE)
    processor_false = SanitizerProcessor(mode=False)

    result_soup_none = processor_none.process(soup)
    result_soup_false = processor_false.process(soup)

    assert str(result_soup_none) == html_input
    assert str(result_soup_false) == html_input


def test_sanitizer_mode_all_html():
    """
    Проверяет, что в режиме SANITIZE_ALL_HTML удаляются все теги.
    """
    html_input = '<p>Hello <b>world</b>! <a href="#">Click me</a>.</p>'
    soup = BeautifulSoup(html_input, 'html.parser')
    processor = SanitizerProcessor(mode=SANITIZE_ALL_HTML)

    result_text = processor.process(soup)

    assert result_text == "Hello world! Click me."


ETPGRF_SANITIZE_TEST_CASES = [
    # ID, Описание, Входной HTML, Ожидаемый HTML
    (
        "simple_unwrap", "Простое разворачивание span'а с одним etp-классом",
        '<p><span class="etp-laquo">«</span>Hello</p>',
        '<p>«Hello</p>'
    ),
    (
        "aggressive_unwrap", "Агрессивное разворачивание span'а со смешанными классами",
        '<p>Hello<span class="user-class etp-raquo">»</span></p>',
        '<p>Hello»</p>'
    ),
    (
        "keep_user_span", "Не трогаем span'ы с пользовательскими классами",
        '<p>Hello <span class="user-class">world</span></p>',
        '<p>Hello <span class="user-class">world</span></p>'
    ),
    (
        "keep_user_span", "Не трогаем span'ы с пользовательскими etp-классами",
        '<p>Hello <span class="etp-user-class">world</span></p>',
        '<p>Hello <span class="etp-user-class">world</span></p>'
    ),
    (
        "keep_other_tags", "Не трогаем другие теги",
        '<div><b>Bold</b> and <i>italic</i></div>',
        '<div><b>Bold</b> and <i>italic</i></div>'
    ),
    (
        "complex_case", "Сложный случай с несколькими разными span'ами",
        '<h1><span class="etp-laquo">«</span>Title<span class="etp-raquo">»</span></h1>\n<p>And <span class="note">note</span>.</p>',
        '<h1>«Title»</h1>\n<p>And <span class="note">note</span>.</p>'
    ),
]

@pytest.mark.parametrize("case_id, description, html_input, expected_html", ETPGRF_SANITIZE_TEST_CASES)
def test_sanitizer_mode_etpgrf(case_id, description, html_input, expected_html):
    """
    Проверяет, что в режиме SANITIZE_ETPGRF удаляется только разметка висячей пунктуации.
    """
    soup = BeautifulSoup(html_input, 'html.parser')
    processor = SanitizerProcessor(mode=SANITIZE_ETPGRF)

    result_soup = processor.process(soup)

    assert str(result_soup) == expected_html


# etpgrf/layout.py
# Модуль для обработки тире, специальных символов и правил их компоновки.

import regex
import logging
from etpgrf.config import (LANG_RU, LANG_EN, CHAR_NBSP, CHAR_THIN_SP, CHAR_NDASH, CHAR_MDASH, CHAR_HELLIP,
                           CHAR_UNIT_SEPARATOR, DEFAULT_POST_UNITS, DEFAULT_PRE_UNITS, UNIT_MATH_OPERATORS,
                           ABBR_COMMON_FINAL, ABBR_COMMON_PREPOSITION)

from etpgrf.comutil import parse_and_validate_langs



# --

# --- Настройки логирования ---
logger = logging.getLogger(__name__)


class LayoutProcessor:
    """
    Обрабатывает тире, псевдографику (например, … -> © и тому подобные) и применяет
    правила расстановки пробелов в зависимости от языка (для тире язык важен, так как.
    Правила типографики различаются для русского и английского языков).
    Предполагается, что на вход уже поступает текст с правильными типографскими
    символами тире (— и –).
    """

    def __init__(self,
                 langs: str | list[str] | tuple[str, ...] | frozenset[str] | None = None,
                 process_initials_and_acronyms: bool = True,
                 process_units: bool | str | list[str] = True):

        self.langs = parse_and_validate_langs(langs)
        self.main_lang = self.langs[0] if self.langs else LANG_RU
        self.process_initials_and_acronyms = process_initials_and_acronyms
        self.process_units = process_units
        # 1. Паттерн для длинного (—) или среднего (–) тире, окруженного пробелами.
        # (?<=[\p{L}\p{Po}\p{Pf}"\']) - просмотр назад на букву, пунктуацию или кавычку.
        self._dash_pattern = regex.compile(rf'(?<=[\p{{L}}\p{{Po}}\p{{Pf}}"\'])\s+([{CHAR_MDASH}{CHAR_NDASH}])\s+(?=\S)')

        # 2. Паттерн для многоточия, за которым следует пробел и слово.
        # Ставит неразрывный пробел после многоточия, чтобы не отрывать его от следующего слова.
        # (?=[\p{L}\p{N}]) - просмотр вперед на букву или цифру.
        self._ellipsis_pattern = regex.compile(rf'({CHAR_HELLIP})\s+(?=[\p{{L}}\p{{N}}])')

        # 3. Паттерн для отрицательных чисел.
        # Ставит неразрывный пробел перед знаком минус, если за минусом идет цифра (неразрывный пробел
        # заменяет обычный). Это предотвращает "отлипание" знака от числа при переносе строки.
        # (?<!\d) - негативный просмотр назад, чтобы правило не срабатывало для бинарного минуса
        #           в выражениях типа "10 - 5".
        self._negative_number_pattern = regex.compile(r'(?<!\d)\s+-(\d+)')

        # 4. Паттерны для обработки инициалов и акронимов.
        # \p{Lu} - любая заглавная буква в Unicode.

        # Правила для случаев, когда пробел УЖЕ ЕСТЬ (заменяем на неразрывный)
        # Используем ` +` (пробел) вместо `\s+`, чтобы не заменять уже вставленные тонкие пробелы.
        self._initial_to_initial_ws_pattern = regex.compile(r'(\p{Lu}\.) +(?=\p{Lu}\.)')
        self._initial_to_surname_ws_pattern = regex.compile(r'(\p{Lu}\.) +(?=\p{Lu}\p{L}{1,})')
        self._surname_to_initial_ws_pattern = regex.compile(r'(\p{Lu}\p{L}{2,}) +(?=\p{Lu}\.)')

        # Правила для случаев, когда пробела НЕТ (вставляем тонкий пробел)
        self._initial_to_initial_ns_pattern = regex.compile(r'(\p{Lu}\.)(?=\p{Lu}\.)')
        self._initial_to_surname_ns_pattern = regex.compile(r'(\p{Lu}\.)(?=\p{Lu}\p{L}{1,})')

        # Вся логика обработки финальных сокращений перенесена в метод process для надежной итеративной обработки

        # 6. Паттерн, описывающий "число" - арабское (включая десятичные дроби через запятую или точку) ИЛИ римское.
        # Для римских цифр используется \b, чтобы не спутать 'I' с частью слова.
        self._NUMBER_PATTERN = r'(?:\d[\d.,]*|\b[IVXLCDM]+\b)'

        # 7. Паттерны для единиц измерения (простые и составные).
        self._post_units_pattern = None
        self._pre_units_pattern = None
        self._complex_unit_pattern = None
        self._math_unit_pattern = None
        if self.process_units:
            all_post_units = list(DEFAULT_POST_UNITS)
            # Добавляем кастомные единицы, если они есть
            if isinstance(self.process_units, str):
                all_post_units.extend(self.process_units.split())
            elif isinstance(self.process_units, (list, tuple, set)):
                all_post_units.extend(self.process_units)

            # Единая проверка безопасности: удаляем все единицы, содержащие временный разделитель.
            safe_units = [unit for unit in all_post_units if CHAR_UNIT_SEPARATOR not in unit]
            if len(safe_units) != len(all_post_units):
                logger.warning(f"One or more units contained the reserved separator ('{CHAR_UNIT_SEPARATOR}') and were ignored.")

            # Создаем паттерны только из безопасных единиц
            if safe_units:
                sorted_units = sorted(safe_units, key=len, reverse=True)
                units_pattern_part_full = '|'.join(map(regex.escape, sorted_units))
                units_pattern_part_clean = '|'.join(map(regex.escape, [u.replace('.', '') for u in sorted_units]))

                # Простые единицы: число + единица
                self._post_units_pattern = regex.compile(rf'({self._NUMBER_PATTERN})\s+({units_pattern_part_full})(?!\w)')
                # Составные единицы: ищет пару "единица." + "единица"
                self._complex_unit_pattern = regex.compile(r'\b(' + units_pattern_part_clean + r')\.(\s*)('
                                                           + units_pattern_part_clean + r')(?!\w)')
                # Математические операции между единицами
                math_ops_pattern = '|'.join(map(regex.escape, UNIT_MATH_OPERATORS))
                self._math_unit_pattern = regex.compile(
                    r'\b(' + units_pattern_part_clean + r')\s*(' + math_ops_pattern + r')\s*('
                    + units_pattern_part_clean + r')(?!\w)')

            # Паттерн для пред-позиционных единиц
            self._pre_units_pattern = regex.compile(
                r'(?<![\p{L}\p{N}])(' + '|'.join(map(regex.escape, DEFAULT_PRE_UNITS)) + rf')\s+({self._NUMBER_PATTERN})')

        logger.debug(f"LayoutProcessor `__init__`. "
                     f"Langs: {self.langs}, "
                     f"Main lang: {self.main_lang}, "
                     f"Process initials and acronyms: {self.process_initials_and_acronyms}, "
                     f"Process units: {bool(self.process_units)}")

    def _replace_dash_spacing(self, match: regex.Match) -> str:
        """Callback-функция для расстановки пробелов вокруг тире с учетом языка."""
        dash = match.group(1)  # Получаем сам символ тире (— или –)
        if self.main_lang == LANG_EN:
            # Для английского языка — слитно, без пробелов.
            return dash
        # По умолчанию (и для русского) — отбивка пробелами.
        return f'{CHAR_NBSP}{dash} '

    def _process_abbreviations(self, text: str, abbreviations: list[str], mode: str) -> str:
        """
        Универсальный обработчик для разных типов сокращений.

        :param text: Входной текст.
        :param abbreviations: Список сокращений для обработки.
        :param mode: 'final' (NBSP ставится перед) или 'prepositional' (NBSP ставится после).
        :return: Обработанный текст.
        """
        processed_text = text

        # Шаг 1: "Склеиваем" многосоставные сокращения временным разделителем CHAR_UNIT_SEPARATOR
        for abbr in sorted(abbreviations, key=len, reverse=True):
            if ' ' in abbr:
                pattern = regex.escape(abbr).replace(r'\ ', r'\s*')
                replacement = abbr.replace(' ', CHAR_UNIT_SEPARATOR)
                processed_text = regex.sub(pattern, replacement, processed_text, flags=regex.IGNORECASE)

        # Шаг 2: Ставим неразрывный пробел.
        glued_abbrs = [a.replace(' ', CHAR_UNIT_SEPARATOR) for a in abbreviations]
        all_abbrs_pattern = '|'.join(map(regex.escape, sorted(glued_abbrs, key=len, reverse=True)))

        if mode == 'final':
            # Ставим nbsp перед сокращением, если перед ним есть пробел
            nbsp_pattern = regex.compile(r'(\s)(' + all_abbrs_pattern + r')(?=[.,!?]|\s|$)', flags=regex.IGNORECASE)
            processed_text = nbsp_pattern.sub(fr'{CHAR_NBSP}\2', processed_text)
        elif mode == 'prepositional':
            # Ставим nbsp после сокращения, если после него есть пробел
            nbsp_pattern = regex.compile(r'(' + all_abbrs_pattern + r')(\s)', flags=regex.IGNORECASE)
            processed_text = nbsp_pattern.sub(fr'\1{CHAR_NBSP}', processed_text)

        # Шаг 3: Заменяем временный разделитель на правильную тонкую шпацию
        return processed_text.replace(CHAR_UNIT_SEPARATOR, CHAR_THIN_SP)

    def process(self, text: str) -> str:
         """Применяет правила компоновки к тексту."""
         # Порядок применения правил важен.
         processed_text = text

         # 1. Обработка пробелов вокруг тире.
         processed_text = self._dash_pattern.sub(self._replace_dash_spacing, processed_text)

         # 2. Обработка пробела после многоточия.
         processed_text = self._ellipsis_pattern.sub(f'\\1{CHAR_NBSP}', processed_text)

         # 3. Обработка пробела перед отрицательными числами/минусом.
         processed_text = self._negative_number_pattern.sub(f'{CHAR_NBSP}-\\1', processed_text)

         # 4. Обработка сокращений.
         processed_text = self._process_abbreviations(processed_text, ABBR_COMMON_FINAL, 'final')
         processed_text = self._process_abbreviations(processed_text, ABBR_COMMON_PREPOSITION, 'prepositional')

         # 5. Обработка инициалов и акронимов (если включено).
         if self.process_initials_and_acronyms:
             # Сначала вставляем тонкие пробелы там, где пробелов не было.
             processed_text = self._initial_to_initial_ns_pattern.sub(f'\\1{CHAR_THIN_SP}', processed_text)
             processed_text = self._initial_to_surname_ns_pattern.sub(f'\\1{CHAR_THIN_SP}', processed_text)

             # Затем заменяем существующие пробелы на неразрывные.
             processed_text = self._initial_to_initial_ws_pattern.sub(f'\\1{CHAR_NBSP}', processed_text)
             processed_text = self._initial_to_surname_ws_pattern.sub(f'\\1{CHAR_NBSP}', processed_text)
             processed_text = self._surname_to_initial_ws_pattern.sub(f'\\1{CHAR_NBSP}', processed_text)

         # 6. Обработка единиц измерения (если включено).
         if self.process_units:
             if self._complex_unit_pattern:
                # Шаг 1: "Склеиваем" все составные единицы с помощью временного разделителя.
                # Цикл безопасен, так как мы заменяем пробелы на непробельный символ, и паттерн не найдет себя снова.
                while self._complex_unit_pattern.search(processed_text):
                    processed_text = self._complex_unit_pattern.sub(
                        fr'\1.{CHAR_UNIT_SEPARATOR}\3', processed_text, count=1)

             if self._math_unit_pattern:
                 # processed_text = self._math_unit_pattern.sub(r'\1/\2', processed_text)
                 processed_text = self._math_unit_pattern.sub(r'\1\2\3', processed_text)
             # И только потом привязываем простые единицы к числам
             if self._post_units_pattern:
                 processed_text = self._post_units_pattern.sub(f'\\1{CHAR_NBSP}\\2', processed_text)
             if self._pre_units_pattern:
                 processed_text = self._pre_units_pattern.sub(f'\\1{CHAR_NBSP}\\2', processed_text)

             # Шаг 2: Заменяем все временные разделители на правильную тонкую шпацию.
             processed_text = processed_text.replace(CHAR_UNIT_SEPARATOR, CHAR_THIN_SP)

         return processed_text



# etpgrf/hyphenation.py
# Представленные здесь алгоритмы реализуют упрощенные правила. Но эти правила лучше, чем их полное отсутствие.
# Тем более что пользователь может отключить переносы из типографа.
# Для русского языка правила реализованы лучше. Для английского дают "разумные" переносы во многих случаях, но из-за
# большого числа беззвучных согласных и их сочетаний, могут давать не совсем корректный результат.

import regex
import logging
import html
from etpgrf.config import (
    CHAR_SHY, LANG_RU, LANG_RU_OLD, LANG_EN,
    RU_VOWELS_UPPER, RU_CONSONANTS_UPPER, RU_J_SOUND_UPPER, RU_SIGNS_UPPER, # RU_ALPHABET_UPPER,
    EN_VOWELS_UPPER, EN_CONSONANTS_UPPER # , EN_ALPHABET_UPPER
)
from etpgrf.defaults import etpgrf_settings
from etpgrf.comutil import parse_and_validate_langs, is_inside_unbreakable_segment


_RU_OLD_VOWELS_UPPER = frozenset(['І',      # И-десятеричное (гласная)
                                  'Ѣ',      # Ять (гласная)
                                  'Ѵ'])     # Ижица (может быть и гласной, и согласной - сложный случай!)
_RU_OLD_CONSONANTS_UPPER = frozenset(['Ѳ',],)   # Фита (согласная)

_EN_SUFFIXES_WITHOUT_HYPHENATION_UPPER = frozenset([
        "ATION", "ITION", "UTION", "OSITY",   # 5-символьные, типа: creation, position, solution, generosity
        "ABLE", "IBLE", "MENT", "NESS",       # 4-символьные, типа: readable, visible, development, kindness
        "LESS", "SHIP", "HOOD", "TIVE",       #                     fearless, friendship, childhood, active (спорно)
        "SION", "TION",                       #                     decision, action (часто покрываются C-C или V-C-V)
        # "ING", "ED", "ER", "EST", "LY"      # совсем короткие, но распространенные, не рассматриваем.
])
_EN_UNBREAKABLE_X_GRAPHS_UPPER = frozenset(["SH", "CH", "TH", "PH", "WH", "CK", "NG", "AW",   # диграфы с согласными
                                         "TCH", "DGE", "IGH",               # триграфы
                                         "EIGH", "OUGH"])                   # квадрографы


# --- Настройки логирования ---
logger = logging.getLogger(__name__)


# --- Класс Hyphenator (расстановка переносов) ---
class Hyphenator:
    """Правила расстановки переносов для разных языков.
    """
    def __init__(self,
                 langs: str | list[str] | tuple[str, ...] | frozenset[str] | None = None,
                 max_unhyphenated_len: int | None = None,  # Максимальная длина непереносимой группы
                 min_tail_len: int | None = None):  # Минимальная длина после переноса (хвост, который разрешено переносить)
        self.langs: frozenset[str] = parse_and_validate_langs(langs)
        self.max_unhyphenated_len = etpgrf_settings.hyphenation.MAX_UNHYPHENATED_LEN if max_unhyphenated_len is None else max_unhyphenated_len
        self.min_chars_per_part = etpgrf_settings.hyphenation.MIN_TAIL_LEN if min_tail_len is None else min_tail_len
        if self.min_chars_per_part < 2:
            # Минимальная длина хвоста должна быть >= 2, иначе вылезаем за индекс в английских словах
            raise ValueError(f"etpgrf: минимальная длина хвоста (min_tail_len) должна быть >= 2,"
                             f" а не {self.min_chars_per_part}")
        if self.max_unhyphenated_len <= self.min_chars_per_part:
            # Максимальная длина непереносимой группы должна быть больше минимальной длины хвоста
            raise ValueError(f"etpgrf: максимальная длина непереносимой группы (max_unhyphenated_len) "
                             f"должна быть больше минимальной длины хвоста (min_tail_len), "
                             f"а не {self.max_unhyphenated_len} >= {self.min_chars_per_part}")

        # Внутренние языковые ресурсы, если нужны специфично для переносов
        self._vowels: frozenset = frozenset()
        self._consonants: frozenset = frozenset()
        self._j_sound_upper: frozenset = frozenset()
        self._signs_upper: frozenset = frozenset()
        self._ru_alphabet_upper: frozenset = frozenset()
        self._en_alphabet_upper: frozenset = frozenset()
        # Загружает наборы символов на основе self.langs
        self._load_language_resources_for_hyphenation()

        # ...
        logger.debug(f"Hyphenator `__init__`. Langs: {self.langs},"
                     f" Max unhyphenated_len: {self.max_unhyphenated_len},"
                     f" Min chars_per_part: {self.min_chars_per_part}")

    def _load_language_resources_for_hyphenation(self):
        # Определяем наборы гласных, согласных и т.д. в зависимости языков.
        if LANG_RU in self.langs:
            self._vowels |= RU_VOWELS_UPPER
            self._consonants |= RU_CONSONANTS_UPPER
            self._j_sound_upper |= RU_J_SOUND_UPPER
            self._signs_upper |= RU_SIGNS_UPPER
            self._ru_alphabet_upper |= self._vowels | self._consonants | self._j_sound_upper | self._signs_upper
        if LANG_RU_OLD in self.langs:
            self._vowels |= RU_VOWELS_UPPER | _RU_OLD_VOWELS_UPPER
            self._consonants |= RU_CONSONANTS_UPPER | _RU_OLD_CONSONANTS_UPPER
            self._j_sound_upper |= RU_J_SOUND_UPPER
            self._signs_upper |= RU_SIGNS_UPPER
            self._ru_alphabet_upper |= self._vowels | self._consonants | self._j_sound_upper | self._signs_upper
        if LANG_EN in self.langs:
            self._vowels |= EN_VOWELS_UPPER
            self._consonants |= EN_CONSONANTS_UPPER
            self._en_alphabet_upper |= EN_VOWELS_UPPER | EN_CONSONANTS_UPPER
        # ... и для других языков, если они поддерживаются переносами


    # Проверка гласных букв
    def _is_vow(self, char: str) -> bool:
        return char.upper() in self._vowels


    # Проверка согласных букв
    def _is_cons(self, char: str) -> bool:
        return char.upper() in self._consonants


    # Проверка полугласной буквы "й"
    def _is_j_sound(self, char: str) -> bool:
        return char.upper() in self._j_sound_upper


    # Проверка мягкого/твердого знака
    def _is_sign(self, char: str) -> bool:
        return char.upper() in self._signs_upper


    def hyp_in_word(self, word: str) -> str:
        """ Расстановка переносов в русском слове с учетом максимальной длины непереносимой группы.
        Переносы ставятся половинным делением слова, рекурсивно.

        :param word:      Слово, в котором надо расставить переносы
        :return:          Слово с расставленными переносами
        """
        # 1. ОБЩИЕ ПРОВЕРКИ
        # TODO: возможно, для скорости, надо сделать проверку на пробелы и другие разделители, которых не должно быть
        if not word:
            # Явная проверка на пустую строку
            return ""
        if len(word) <= self.max_unhyphenated_len or not any(self._is_vow(c) for c in word):
            # Если слово короткое или не содержит гласных, перенос не нужен
            return word
        logger.debug(f"Hyphenator: word: `{word}` // langs: {self.langs} // max_unhyphenated_len: {self.max_unhyphenated_len} // min_tail_len: {self.min_chars_per_part}")
        # 2. ОБНАРУЖЕНИЕ ЯЗЫКА И ПОДКЛЮЧЕНИЕ ЯЗЫКОВОЙ ЛОГИКИ
        # Поиск вхождения букв строки (слова) через `frozenset` -- O(1). Это быстрее регулярного выражения -- O(n)
        # 2.1. Проверяем RU и RU_OLD (правила одинаковые, но разные наборы букв)
        if (LANG_RU in self.langs or LANG_RU_OLD in self.langs) and frozenset(word.upper()) <= self._ru_alphabet_upper:
            # Пользователь подключил русскую логику, и слово содержит только русские буквы
            logger.debug(f"`{word}` -- use `{LANG_RU}` or `{LANG_RU_OLD}` rules")

            # Поиск допустимой позиции для переноса около заданного индекса
            def find_hyphen_point_ru(word_segment: str, start_idx: int) -> int:
                word_len = len(word_segment)
                min_part = self.min_chars_per_part

                # --- Вложенная функция для оценки качества точки переноса ---
                def get_split_score(i: int) -> int:
                    """
                    Вычисляет "оценку" для точки переноса `i`. Чем выше оценка, тем качественнее перенос.
                    -1 означает, что перенос в этой точке запрещен.
                    """
                    # --- Сначала идут ЗАПРЕТЫ (жесткие "нельзя") ---
                    # Если правило нарушено, сразу дисквалифицируем точку.
                    if self._is_sign(word_segment[i]) or self._is_j_sound(word_segment[i]):
                        return -1  # ЗАПРЕТ 1: Новая строка не может начинаться с Ь, Ъ или Й.
                    if self._is_j_sound(word_segment[i - 1]) and self._is_vow(word_segment[i]):
                        return -1  # ЗАПРЕТ 2: Нельзя отрывать Й от следующей за ней гласной.
                    # --- Теперь идут РАЗРЕШЕНИЯ с разными приоритетами ---
                    # РАЗРЕШЕНИЕ 1: Перенос между сдвоенными согласными.
                    if self._is_cons(word_segment[i - 1]) and word_segment[i - 1] == word_segment[i]:
                        return 10
                    # РАЗРЕШЕНИЕ 2: Перенос после "слога" с Ь/Ъ, если дальше идет СОГЛАСНАЯ.
                    #               Пример: "строитель-ство", но НЕ "компь-ютер".
                    #               По-хорошему нужно проверять, что перед Ь/Ъ нет йотированной гласной
                    #               (и переработать ЗАПРЕТ 2), но это еще больше усложнит логику.
                    if self._is_sign(word_segment[i - 1]) and self._is_cons(word_segment[i]):
                        return 9
                    # РАЗРЕШЕНИЕ 3: Перенос после "слога" если предыдущий Й (очень качественный перенос).
                    if self._is_j_sound(word_segment[i - 1]):
                        return 7
                    # РАЗРЕШЕНИЕ 4: Перенос между тремя согласными (C-CС), чуть лучше, чем после гласной.
                    if self._is_cons(word_segment[i]) and self._is_cons(word_segment[i-1]) and self._is_cons(word_segment[i+1]):
                        return 6
                    # # РАЗРЕШЕНИЕ 5 (?): Перенос между согласной и согласной (C-C).
                    # if self._is_cons(word_segment[i - 1]) and self._is_cons(word_segment[i]):
                    #     return 5
                    # РАЗРЕШЕНИЕ 6 (Основное правило): Перенос после гласной.
                    if self._is_vow(word_segment[i - 1]):
                        return 5
                    # Если ни одно правило не подошло, точка не подходит для переноса.
                    return 0

                # 1. Собираем всех кандидатов и их оценки
                candidates = []
                possible_indices = range(min_part, word_len - min_part + 1)
                for i in possible_indices:
                    score = get_split_score(i)
                    if score > 0:
                        # Добавляем только подходящих кандидатов
                        distance_from_center = abs(i - start_idx)
                        candidates.append({'score': score, 'distance': distance_from_center, 'index': i})

                # 2. Если подходящих кандидатов нет, сдаемся
                if not candidates:
                    return -1

                # 3. Сортируем кандидатов: сначала по убыванию ОЦЕНКИ, потом по возрастанию УДАЛЕННОСТИ от центра.
                # Это гарантирует, что перенос "н-н" (score=10) будет выбран раньше, чем "е-н" (score=5),
                # даже если "е-н" чуть ближе к центру.
                best_candidate = sorted(candidates, key=lambda c: (-c['score'], c['distance']))[0]

                return best_candidate['index']  # Не нашли подходящую позицию

            # Рекурсивное деление слова
            def split_word_ru(word_to_split: str) -> str:
                # Если длина укладывается в лимит, перенос не нужен
                if len(word_to_split) <= self.max_unhyphenated_len:
                    return word_to_split
                # Ищем точку переноса около середины
                hyphen_idx = find_hyphen_point_ru(word_to_split, len(word_to_split) // 2)
                # Если не нашли точку переноса
                if hyphen_idx == -1:
                    return word_to_split
                # Разделяем слово на две части (до и после точки переноса)
                left_part = word_to_split[:hyphen_idx]
                right_part = word_to_split[hyphen_idx:]
                # Рекурсивно делим левую и правую части и соединяем их через символ переноса
                return split_word_ru(left_part) + CHAR_SHY + split_word_ru(right_part)

            # Основная логика
            return split_word_ru(word)    # Рекурсивно делим слово на части с переносами

        # 2.2. Проверяем EN
        elif LANG_EN in self.langs and frozenset(word.upper()) <= self._en_alphabet_upper:
            # Пользователь подключил английскую логику, и слово содержит только английские буквы
            logger.debug(f"`{word}` -- use `{LANG_EN}` rules")
            # --- Начало логики для английского языка (заглушка) ---
            # ПРИМЕЧАНИЕ: правила переноса в английском языке основаны на слогах, и их точное определение без словаря
            # слогов или сложного алгоритма (вроде Knuth-Liang) — непростая задача. Здесь реализована упрощенная
            # логика и поиск потенциальных точек переноса основан на простых правилах: между согласными, или между
            # гласной и согласной. Метод половинного деления и рекурсии (поиск переносов о середины слова).

            # Функция для поиска допустимой позиции для переноса около заданного индекса
            # Ищет точку переноса, соблюдая min_chars_per_part и простые правила
            def find_hyphen_point_en(word_segment: str, start_idx: int) -> int:
                word_len = len(word_segment)
                min_part = self.min_chars_per_part

                # Определяем диапазон допустимых индексов для переноса
                # Индекс 'i' - это точка разреза. word_segment[:i] и word_segment[i:] должны быть не короче min_part.
                # i >= min_part
                # word_len - i >= min_part => i <= word_len - min_part
                valid_split_indices = [i for i in range(min_part, word_len - min_part + 1)]

                if not valid_split_indices:
                    # Нет ни одного места, где можно поставить перенос, соблюдая min_part
                    logger.debug(f"No valid split indices for '{word_segment}' within min_part={min_part}")
                    return -1

                # Сортируем допустимые индексы по удаленности от start_idx (середины)
                # Это реализует поиск "около центра"
                valid_split_indices.sort(key=lambda i: abs(i - start_idx))

                # Проверяем каждый потенциальный индекс переноса по упрощенным правилам
                for i in valid_split_indices:
                    # Упрощенные правила английского переноса (основаны на частых паттернах, не на слогах):
                    # 1. Запрет переноса между гласными
                    if self._is_vow(word_segment[i - 1]) and self._is_vow(word_segment[i]):
                        logger.debug(
                            f"Skipping V-V split point at index {i} in '{word_segment}' ({word_segment[i - 1]}{word_segment[i]})")
                        continue  # Переходим к следующему кандидату i

                    # 2. Запрет переноса ВНУТРИ неразрывных диграфов/триграфов и т.д.
                    if is_inside_unbreakable_segment(word_segment=word_segment,
                                                     split_index=i,
                                                     unbreakable_set=_EN_UNBREAKABLE_X_GRAPHS_UPPER):
                        logger.debug(f"Skipping unbreakable segment at index {i} in '{word_segment}'")
                        continue

                    # 3. Перенос между двумя согласными (C-C), например, 'but-ter', 'subjec-tive'
                    #    Точка переноса - индекс i. Проверяем символы word[i-1] и word[i].
                    if self._is_cons(word_segment[i - 1]) and self._is_cons(word_segment[i]):
                        logger.debug(f"Found C-C split point at index {i} in '{word_segment}'")
                        return i

                    # 4. Перенос перед одиночной согласной между двумя гласными (V-C-V), например, 'ho-tel', 'ba-by'
                    #    Точка переноса - индекс i (перед согласной). Проверяем word[i-1], word[i], word[i+1].
                    #    Требуется как минимум 3 символа для этого паттерна.
                    if i < word_len - 1 and \
                            self._is_vow(word_segment[i - 1]) and self._is_cons(word_segment[i]) and self._is_vow(
                        word_segment[i + 1]):
                        logger.debug(f"Found V-C-V (split before C) split point at index {i} in '{word_segment}'")
                        return i

                    # 5. Перенос после одиночной согласной между двумя гласными (V-C-V), например, 'riv-er', 'fin-ish'
                    #    Точка переноса - индекс i (после согласной). Проверяем word[i-2], word[i-1], word[i].
                    #    Требуется как минимум 3 символа для этого паттерна.
                    if i < word_len and \
                            self._is_vow(word_segment[i - 2]) and self._is_cons(word_segment[i - 1]) and \
                            self._is_vow(word_segment[i]):
                        logger.debug(f"Found V-C-V (split after C) split point at index {i} in '{word_segment}'")
                        return i

                    # 6. Правила для распространенных суффиксов (перенос ПЕРЕД суффиксом). Проверяем, что word_segment
                    #    заканчивается на суффикс, и точка переноса (i) находится как раз перед ним
                    if word_segment[i:].upper() in _EN_SUFFIXES_WITHOUT_HYPHENATION_UPPER:
                        # Мы нашли потенциальный суффикс.
                        logger.debug(f"Found suffix '-{word_segment[i:]}' split point at index {i} in '{word_segment}'")
                        return i

                # Если ни одна подходящая точка переноса не найдена в допустимом диапазоне
                logger.debug(f"No suitable hyphen point found for '{word_segment}' near center.")
                return -1

            # Рекурсивная функция для деления слова на части с переносами
            def split_word_en(word_to_split: str) -> str:
                # Базовый случай рекурсии: если часть слова достаточно короткая, не делим ее дальше
                if len(word_to_split) <= self.max_unhyphenated_len:
                    return word_to_split

                # Ищем точку переноса около середины текущей части слова
                hyphen_idx = find_hyphen_point_en(word_to_split, len(word_to_split) // 2)

                # Если подходящая точка переноса не найдена, возвращаем часть слова как есть
                if hyphen_idx == -1:
                    return word_to_split

                # Рекурсивно обрабатываем обе части и объединяем их символом переноса
                return (split_word_en(word_to_split[:hyphen_idx]) +
                        CHAR_SHY + split_word_en(word_to_split[hyphen_idx:]))

            # --- Конец логики для английского языка ---
            return split_word_en(word)
        else:
            # кстати "слова" в которых есть пробелы или другие разделители, тоже попадают сюда
            logger.debug(f"`{word}` -- use `UNDEFINE` rules")
            return word


    def hyp_in_text(self, text: str) -> str:
        """ Расстановка переносов в тексте

            :param text: Строка, которую надо обработать (главный аргумент).
            :return: str: Строка с расставленными переносами.
        """

        # 1. Определяем функцию, которая будет вызываться для каждого найденного слова
        def replace_word_with_hyphenated(match_obj):
            # Модуль regex автоматически передает сюда match_obj для каждого совпадения.
            # Чтобы получить `слово` из 'совпадения' делаем .group() или .group(0).
            word_to_process = match_obj.group(0)
            # И оправляем это слово на расстановку переносов (внутри hyp_in_word уже есть все проверки).
            hyphenated_word = self.hyp_in_word(word_to_process)

            # ============= Для отладки (слова в которых появились переносы) ==================
            if word_to_process != hyphenated_word:
                logger.debug(f"hyp_in_text: '{word_to_process}' -> '{hyphenated_word}'")

            return hyphenated_word

        # 2. regex.sub() -- поиск с заменой. Ищем по паттерну `r'\b\p{L}+\b'`  (`\b` - граница слова;
        #                   `\p{L}` - любая буква Unicode; `+` - одно или более вхождений).
        #                    Второй аргумент - это наша функция replace_word_with_hyphenated.
        #                    regex.sub вызовет ее для каждого найденного слова, передав match_obj.
        processed_text = regex.sub(pattern=r'\b\p{L}+\b', repl=replace_word_with_hyphenated, string=text)

        return processed_text





# etpgrf/symbols.py
# Модуль для преобразования псевдографики в правильные типографские символы.

import regex
import logging
from .config import CHAR_NDASH, STR_TO_SYMBOL_REPLACEMENTS

logger = logging.getLogger(__name__)


class SymbolsProcessor:
    """
    Преобразует ASCII-последовательности (псевдографику) в семантически
    верные Unicode-символы. Работает на раннем этапе, до расстановки пробелов.
    """

    def __init__(self):
        # Для сложных замен, требующих анализа контекста (например, диапазоны),
        # по-прежнему используем регулярные выражения.
        # Паттерн для диапазонов: цифра-дефис-цифра -> цифра–цифра (среднее тире).
        # Обрабатываем арабские и римские цифры.
        self._range_pattern = regex.compile(pattern=r'(\d)-(\d)|([IVXLCDM]+)-([IVXLCDM]+)', flags=regex.IGNORECASE)

        logger.debug("SymbolsProcessor `__init__`")

    def _replace_range(self, match: regex.Match) -> str:
        # Паттерн имеет две группы: (\d)-(\d) ИЛИ ([IVX...])-([IVX...])
        if match.group(1) is not None:  # Арабские цифры
            return f'{match.group(1)}{CHAR_NDASH}{match.group(2)}'
        if match.group(3) is not None:  # Римские цифры
            return f'{match.group(3)}{CHAR_NDASH}{match.group(4)}'
        return match.group(0)  # На всякий случай


    def process(self, text: str) -> str:
        # Шаг 1: Выполняем простые замены из списка `STR_TO_SYMBOL_REPLACEMENTS` (см. config.py).
        # Этот шаг должен идти первым, чтобы пользователь мог, например,
        # использовать '---' в диапазоне '1---5', если ему это нужно.
        # В таком случае '---' заменится на '—', и правило для диапазонов
        # с дефисом уже не сработает.
        processed_text = text
        for old, new in STR_TO_SYMBOL_REPLACEMENTS:
            processed_text = processed_text.replace(old, new)

        # Шаг 2: Обрабатываем диапазоны с помощью регулярного выражения.
        # Эта замена более специфична и требует контекста (цифры вокруг дефиса).
        processed_text = self._range_pattern.sub(self._replace_range, processed_text)

        return processed_text




# tests/test_typograph.py
# Тестирует основной класс Typographer и его конвейер обработки.

import pytest
from etpgrf import Typographer
from etpgrf.config import CHAR_NBSP, CHAR_THIN_SP, CHAR_NDASH, CHAR_MDASH, SANITIZE_ETPGRF, SANITIZE_ALL_HTML

TYPOGRAPHER_HTML_TEST_CASES = [
    # --- Базовая обработка без HTML ---
    ('mnemonic', 'Простой текст с "кавычками".',  f'Простой текст с&nbsp;&laquo;кавычками&raquo;.'),
    ('mixed', 'Простой текст с "кавычками".', f'Простой текст с&nbsp;«кавычками».'),
    ('unicode', 'Простой текст с "кавычками".',  f'Простой текст с{CHAR_NBSP}«кавычками».'),
    # --- Базовая обработка с HTML ---
    ('mnemonic', '<p>Простой параграф с «кавычками».</p>', '<p>Простой параграф с&nbsp;&laquo;кавычками&raquo;.</p>'),
    ('mixed', '<p>Простой параграф с "кавычками".</p>', '<p>Простой параграф с&nbsp;«кавычками».</p>'),
    ('unicode', '<p>Простой параграф с "кавычками".</p>', f'<p>Простой параграф с{CHAR_NBSP}«кавычками».</p>'),
    # --- Рекурсивный обход ---
    ('mnemonic', '<div><p>Текст, а внутри <b>для проверки "жирный"</b> текст.</p></div>',
                 '<div><p>Текст, а&nbsp;внутри <b>для&nbsp;проверки &laquo;жирный&raquo;</b> текст.</p></div>'),
    ('mixed', '<div><p>Текст, а внутри <b>для проверки "жирный"</b> текст.</p></div>',
              '<div><p>Текст, а&nbsp;внутри <b>для&nbsp;проверки «жирный»</b> текст.</p></div>'),
    ('unicode', '<div><p>Текст, а внутри <b>для проверки "жирный"</b> текст.</p></div>',
                f'<div><p>Текст, а{CHAR_NBSP}внутри <b>для{CHAR_NBSP}проверки «жирный»</b> текст.</p></div>'),
    # --- Вложенные теги с предлогом в тексте ---
    ('mnemonic', '<div><p>Текст с предлогом <b>в <i>доме</i></b>.</p></div>',
                 '<div><p>Текст с&nbsp;предлогом <b>в&nbsp;<i>доме</i></b>.</p></div>'),
    ('mixed', '<div><p>Текст с предлогом <b>в <i>доме</i></b>.</p></div>',
              '<div><p>Текст с&nbsp;предлогом <b>в&nbsp;<i>доме</i></b>.</p></div>'),
    ('unicode', '<div><p>Текст с предлогом <b>в <i>доме</i></b>.</p></div>',
                f'<div><p>Текст с{CHAR_NBSP}предлогом <b>в{CHAR_NBSP}<i>доме</i></b>.</p></div>'),
    # --- Обработка соседних текстовых узлов ---
    ('mnemonic', '<p>Союз и <b>слово</b> и еще один союз а <span>текст</span>.</p>',
                 '<p>Союз и&nbsp;<b>слово</b> и&nbsp;еще один союз а&nbsp;<span>текст</span>.</p>'),
    ('mixed', '<p>Союз и <b>слово</b> и еще один союз а <span>текст</span>.</p>',
              '<p>Союз и&nbsp;<b>слово</b> и&nbsp;еще один союз а&nbsp;<span>текст</span>.</p>'),
    ('unicode', '<p>Союз и <b>слово</b> и еще один союз а <span>текст</span>.</p>',
                f'<p>Союз и{CHAR_NBSP}<b>слово</b> и{CHAR_NBSP}еще один союз а{CHAR_NBSP}<span>текст</span>.</p>'),

    # --- Проверка тегов <style>, <script>, <pre>, <code>, <kbd<, <samp> и <math> ---
    ('mixed', '<p>Текст "до".</p><pre>  - 10</pre><code>"тоже не трогать"</code>',
              '<p>Текст «до».</p><pre>  - 10</pre><code>"тоже не трогать"</code>'),
    ('mixed', '<p>Текст "до".</p><style>body { font-family: "Arial"; }</style>',
              '<p>Текст «до».</p><style>body { font-family: "Arial"; }</style>'),
    ('mixed', '<p>Текст "до".</p><script>var text = "не трогать";</script>',
                '<p>Текст «до».</p><script>var text = "не трогать";</script>'),
    ('mixed', '<p>Текст "до".</p><kbd>Ctrl + C</kbd>',
              '<p>Текст «до».</p><kbd>Ctrl + C</kbd>'),
    ('mixed', '<p>Текст "до".</p><samp>Sample "text"</samp>',
              '<p>Текст «до».</p><samp>Sample "text"</samp>'),
    ('mixed', '<p>Текст "до".</p><math><mi>x</mi><mo>=</mo><mn>5</mn></math>',
              '<p>Текст «до».</p><math><mi>x</mi><mo>=</mo><mn>5</mn></math>'),

    # --- Проверка тегов с атрибутами ---
    ('mixed', '<a href="/a-b" title="Текст в кавычках \'внутри\' атрибута">Текст "снаружи"</a>',
              '<a href="/a-b" title="Текст в кавычках \'внутри\' атрибута">Текст «снаружи»</a>'),
    ('mixed', '<a href="/a-b" title=\'Текст в кавычках \"внутри\" атрибута\'>Текст "снаружи"</a>',
              '<a href="/a-b" title=\'Текст в кавычках \"внутри\" атрибута\'>Текст «снаружи»</a>'),
    ('mixed', '<a href="/a-b" title="Текст в кавычках &laquo;внутри&raquo; атрибута">Текст "снаружи"</a>',
              '<a href="/a-b" title="Текст в кавычках «внутри» атрибута">Текст «снаружи»</a>'),
    ('mnemonic', '<a href="/a-b" title="Текст в кавычках &laquo;внутри&raquo; атрибута">Текст "снаружи"</a>',
                 '<a href="/a-b" title="Текст в кавычках «внутри» атрибута">Текст &laquo;снаружи&raquo;</a>'),

    # --- Комплексный интеграционный тест ---
    ('mnemonic', '<p>Он сказал: "В 1941-1945 гг. -- было 100 тыс. руб. и т. д."</p>',
                 '<p>Он&nbsp;сказал: &laquo;В&nbsp;1941&ndash;1945&nbsp;гг.&nbsp;&ndash; было 100&nbsp;тыс.&thinsp;руб.'
                 ' и&nbsp;т.&thinsp;д.&raquo;</p>'),
    ('mixed', '<p>Он сказал: "В 1941-1945 гг. -- было 100 тыс. руб. и т. д."</p>',
              '<p>Он&nbsp;сказал: «В&nbsp;1941–1945&nbsp;гг.&nbsp;– было 100&nbsp;тыс.&thinsp;руб.'
              ' и&nbsp;т.&thinsp;д.»</p>'),
    ('unicode', '<p>Он сказал: "В 1941-1945 гг. -- было 100 тыс. руб. и т. д."</p>',
                f'<p>Он{CHAR_NBSP}сказал: «В{CHAR_NBSP}1941{CHAR_NDASH}1945{CHAR_NBSP}гг.{CHAR_NBSP}{CHAR_NDASH} было'
                f' 100{CHAR_NBSP}тыс.{CHAR_THIN_SP}руб. и{CHAR_NBSP}т.{CHAR_THIN_SP}д.»</p>'),
    # --- Теги внутри кавычек ---
    ('mnemonic', '<p>"<u>Почему</u>", "<u>зачем</u>" и "<u>кому это выгодно</u>" -- вопросы требующие ответа.</p>',
                 '<p>&laquo;<u>Почему</u>&raquo;, &laquo;<u>зачем</u>&raquo; и&nbsp;&laquo;<u>кому это выгодно</u>'
                 '&raquo;&nbsp;&ndash; вопросы требующие ответа.</p>'),
    ('mixed', '<p>"<u>Почему</u>", "<u>зачем</u>" и "<u>кому это выгодно</u>" -- вопросы требующие ответа.</p>',
                '<p>«<u>Почему</u>», «<u>зачем</u>» и&nbsp;«<u>кому это выгодно</u>»&nbsp;– вопросы требующие ответа.</p>'),
    ('unicode', '<p>"<u>Почему</u>", "<u>зачем</u>" и "<u>кому это выгодно</u>" -- вопросы требующие ответа.</p>',
                f'<p>«<u>Почему</u>», «<u>зачем</u>» и{CHAR_NBSP}«<u>кому это выгодно</u>»{CHAR_NBSP}{CHAR_NDASH} вопросы требующие ответа.</p>'),

    # --- Проверка пустого текста и узлов с пробелами ---
    ('mnemonic', '<p>  </p><div>\n\t</div><p>Слово</p>', '<p> </p><div>\n</div><p>Слово</p>'),
    ('mixed', '<p>  </p><div>\n\t</div><p>Слово</p>', '<p> </p><div>\n</div><p>Слово</p>'),
    ('unicode', '<p>  </p><div>\n\t</div><p>Слово</p>', '<p> </p><div>\n</div><p>Слово</p>'),

    # --- Самозакрывающиеся теги и теги с атрибутами ---
    #     ВАЖНО: 1. Порядок атрибутов в типографированном тексте может быть произвольным
    #            2. Любое число пробельных символов внутри "пустых" тегов будут редуцированы до одного пробела или
    #               перевода строки.
    #            3. Самозакрывающиеся теги будут приведены к единому виду с косой чертой в конце. Типа <br/>
    #            4. Все это "проделки" связаны с использованием библиотеки BeautifulSoup для парсинга HTML,
    #               так что может произойти и другой "улучшайзинг".
    ('mnemonic', '<p>Текст с картинкой <img src="image.jpg" alt="image" /> и текстом.</p>',
                 '<p>Текст с&nbsp;картинкой <img alt="image" src="image.jpg"/> и&nbsp;текстом.</p>'),
    ('mnemonic', '<p>Текст с <code>&lt;br&gt;</code><br>А это новая строка.</p>',
                 '<p>Текст с&nbsp;<code>&lt;br&gt;</code><br/>А&nbsp;это новая строка.</p>'),
    ('mixed', '<p>Текст с картинкой <img src="image.jpg" alt="image" /> и текстом.</p>',
              '<p>Текст с&nbsp;картинкой <img alt="image" src="image.jpg"/> и&nbsp;текстом.</p>'),
    ('mixed', '<p>Текст с <code>&lt;br&gt;</code><br>А это новая строка.</p>',
              '<p>Текст с&nbsp;<code>&lt;br&gt;</code><br/>А&nbsp;это новая строка.</p>'),
    ('unicode', '<p>Текст с картинкой <img src="image.jpg" alt="image" /> и текстом.</p>',
                f'<p>Текст с{CHAR_NBSP}картинкой <img alt="image" src="image.jpg"/> и{CHAR_NBSP}текстом.</p>'),
    ('unicode', '<p>Текст с <code>&lt;br&gt;</code><br>А это новая строка.</p>',
                f'<p>Текст с{CHAR_NBSP}<code>&lt;br&gt;</code><br/>А{CHAR_NBSP}это новая строка.</p>'),
]


@pytest.mark.parametrize("mode, input_html, expected_html", TYPOGRAPHER_HTML_TEST_CASES)
def test_typographer_html_processing(mode, input_html, expected_html):
    """
    Проверяет полный конвейер Typographer при обработке HTML.
    """
    typo = Typographer(langs='ru', process_html=True, mode=mode)
    actual_html = typo.process(input_html)
    assert actual_html == expected_html


def test_typographer_plain_text_processing():
    """
    Проверяет, что в режиме process_html=False типограф маскирует HTML-теги и обрабатывает весь текст.
    """
    typo = Typographer(langs='ru', process_html=False)
    input_text = '<i>Текст "без" <b>HTML</b>, но с предлогом в доме.</i>'
    expected_text = '&lt;i&gt;Текст «без» &lt;b&gt;HTML&lt;/b&gt;, но&nbsp;с&nbsp;предлогом в&nbsp;доме.&lt;/i&gt;'
    actual_text = typo.process(input_text)
    assert actual_text == expected_text


def test_typographer_sanitizer_etpgrf_integration():
    """
    Интеграционный тест: проверяет, что Typographer вызывает Sanitizer для очистки ETP-разметки.
    """
    input_html = '<p>Текст со <span class="etp-laquo">"старой"</span> разметкой.</p>'
    # Ожидаем, что "старая" разметка будет удалена, а "новая" (кавычки-елочки) будет добавлена.
    expected_html = '<p>Текст со&nbsp;«старой» разметкой.</p>'
    typo = Typographer(langs='ru', process_html=True, sanitizer=SANITIZE_ETPGRF, mode='mixed')
    actual_html = typo.process(input_html)
    assert actual_html == expected_html


def test_typographer_sanitizer_all_html_integration():
    """
    Интеграционный тест: проверяет, что Typographer вызывает Sanitizer для полной очистки HTML.
    """
    input_html = '<p>Текст с "кавычками" и <b>жирным</b> текстом.</p>'
    # Ожидаем, что все теги будут удалены, а к чистому тексту применится типографика.
    expected_text = 'Текст с&nbsp;«кавычками» и&nbsp;жирным текстом.'
    typo = Typographer(langs='ru', process_html=True, sanitizer=SANITIZE_ALL_HTML, mode='mixed')
    actual_text = typo.process(input_html)
    assert actual_text == expected_text


# --- Новые тесты на структуру HTML (проверка отсутствия лишних оберток) ---
HTML_STRUCTURE_TEST_CASES = [
    # 1. Фрагмент HTML (без html/body) -> должен остаться фрагментом
    ('<div>Текст</div>', '<div>Текст</div>'),
    ('<span>Текст</span>', '<span>Текст</span>'),
    ('<p>Текст</p>', '<p>Текст</p>'),
    
    # 2. Голый текст -> должен остаться голым текстом (без <p>, <html>, <body>)
    ('Текст без тегов', 'Текст без&nbsp;тегов'), # Исправлено: ожидаем nbsp
    ('Текст с <b>тегом</b> внутри', 'Текст с&nbsp;<b>тегом</b> внутри'),

    # 3. Полноценный html-документ -> должен сохранить структуру
    ('<html><body><p>Текст</p></body></html>', '<html><body><p>Текст</p></body></html>'),
    ('<!DOCTYPE html><html><head></head><body><p>Текст</p></body></html>',
     '<!DOCTYPE html><html><head></head><body><p>Текст</p></body></html>'), # BS может добавить перенос строки после doctype

    # 4. Кривой html -> будет "починен"
    ('<div>Текст', '<div>Текст</div>'),
    ('<p>Текст', '<p>Текст</p>'),
    ('Текст <b>жирный <i>курсив', 'Текст <b>жирный <i>курсив</i></b>'),
    # Используем валидный HTML для теста с DOCTYPE
    ('<!DOCTYPE html><html><head><title>Title</title></head><body><p>Текст</p></body></html>',
     '<!DOCTYPE html><html><head><title>Title</title></head><body><p>Текст</p></body></html>'),
    # Тест на совсем кривой HTML (см ниже) не проходит: весь текст после незарытого <title> передается в заголовок.
    # ('<!DOCTYPE html><html><head><title>Title<body><p>Текст', '<!DOCTYPE html><html><head><title>Title</title></head><body><p>Текст</p></body></html>'),
]

@pytest.mark.parametrize("input_html, expected_html", HTML_STRUCTURE_TEST_CASES)
def test_typographer_html_structure_preservation(input_html, expected_html):
    """
    Проверяет, что Typographer не добавляет лишние теги (html, body, p) 
    вокруг фрагментов и текста, но сохраняет их, если они были.
    """
    # Отключаем все "украшательства" (кавычки, неразрывные пробелы), 
    # чтобы проверять только структуру тегов.
    typo = Typographer(
        langs='ru', 
        process_html=True, 
        mode='mixed',
        hyphenation=False,
        quotes=False,
        unbreakables=True, # Оставим unbreakables, чтобы проверить, что &nbsp; добавляются, но теги не ломаются
        layout=False,
        symbols=False
    )
    actual_html = typo.process(input_html)
    
    # Для теста с doctype может быть нюанс с форматированием (переносы строк), 
    # поэтому нормализуем пробелы перед сравнением
    if '<!DOCTYPE' in input_html:
        assert '<html>' in actual_html
        assert '<body>' in actual_html
        assert '<p>Текст</p>' in actual_html
    else:
        assert actual_html == expected_html
